/*****************************************/
/* Demo 2 using zeus 3d rendering engine */
/*****************************************/

/* A better version of MDL_plot_model is now incorporated into zeus
but has been left in to show programmers what it is like to program
zeus at its lowest level (where speed optimisations can be made) */

/* This demo shows how to do the following:-

1) Create and use a device
2) Create and setup viewports
3) Use lighting
4) Use texture mapping
5) How to program zeus at the lowest level
6) Loading 3DS files
7) Use depth queuing
8) Use back face culling
9) Use SONY compatbible geometry functions
10) Use order tables
11) Use CLUTS and texture pages
12) Install and use zeus's custom keyboard handler
13) Using pentium timing functions

*/


#define NUM_MODELS	3

#include <conio.h>
#include <graph.h>


/**********************************/
/**** main bollocks of program ****/
/**********************************/

/****************************/
/**** user include files ****/
/****************************/
#include "allinc.h"

	int shade=MAX_SHADE;
	int	plot_z=-1;

	MATRIX	cmat={
					MAX_SHADE, MAX_SHADE, MAX_SHADE,
					0,0,0,
					0,0,0,
					0,0,0	/* translation (none)*/
				};

	MATRIX	light;

	DVECTOR		sxy[10000];
	short		sz[10000];

/* Similar functions are now included into zeus (models normals are
generally calculated when a 3DS model files are loaded */
void calculate_models_normals(MDL *mdl, SVECTOR *normals)
{
	SVECTOR		*v1, *v2, *v3;
	SVECTOR		side1, side2;
	int			nf;
	SVECTOR		*v;
	u_short		*vil;
	int			c=8;
	

	while (mdl!=NULL)
	{
		nf=mdl->num_faces;
		v=mdl->vertex_list;
		vil=mdl->vertex_index_list;
		while (nf>0)
		{
			v1=v + *(vil);
			v2=v + *(vil+1);
			v3=v + *(vil+2);

			side1.vx=v2->vx - v1->vx;
			side1.vy=v2->vy - v1->vy;
			side1.vz=v2->vz - v1->vz;
			side2.vx=v3->vx - v1->vx;
			side2.vy=v3->vy - v1->vy;
			side2.vz=v3->vz - v1->vz;

			Normal(&side1, &side2, normals);
			Normalize(normals++);

			vil+=TRI;

			nf--;
		};
		mdl->default_color=c;
		c+=16;
		c&=255;
		mdl=mdl->next_model;
	}
}

void MDL_plot_model_z(VIEWPORT_TYPE *vp, MDL *mdl)
{
	int		cull;
	int		ot_pos;
	int		p, flag;
	CVECTOR	color[1]={MAX_SHADE>>1, MAX_SHADE>>1, MAX_SHADE>>1};
	int		uc, vc;
	

	int		nf, nv;
	SVECTOR	*v;
	u_short	*vil;
	SVECTOR	*v1, *v2, *v3;
	DLVECTOR	*uv;


	while (mdl!=NULL)
	{
		nf=mdl->num_faces;
		nv=mdl->num_vertices;
		if (nf>0)
		{
			v=mdl->vertex_list;
			uv=mdl->uv_list;
			vil=mdl->vertex_index_list;

			while (nf>0)
			{
				v1=v + *(vil);
				v2=v + *(vil+1);
				v3=v + *(vil+2);
				cull=RotAverageNclip3(v1, v2, v3, next_vertex_ptr, next_vertex_ptr+1, next_vertex_ptr+2,
								&p, &ot_pos, &flag);
				if (cull<=0)
				{
					next_face_ptr->v=next_vertex_ptr;
					next_face_ptr->face_data=next_index_ptr;
					next_face_ptr->nv=TRI;
					*(next_index_ptr++)=0;
					*(next_index_ptr++)=1;
					*(next_index_ptr++)=2;
		
					if (face_in_view(next_face_ptr)>0)
					{
						next_face_ptr->tag=NULL;
						next_face_ptr->id=poly_type;
						next_face_ptr->tpage=0;
						next_face_ptr->bump_tpage=0;
						next_face_ptr->semi=0;
						next_face_ptr->rgb=0;
						next_face_ptr->c=256-8;
						next_face_ptr->r=color[0].r;
						next_face_ptr->g=color[0].g;
						next_face_ptr->b=color[0].b;

						uc=(uv + *(vil))->vx>>1;
						vc=(uv + *(vil))->vy>>1;
						next_vertex_ptr->u=uc;
						next_vertex_ptr->v=vc;
						next_vertex_ptr++;
						uc=(uv + *(vil+1))->vx>>1;
						vc=(uv + *(vil+1))->vy>>1;
						next_vertex_ptr->u=uc;
						next_vertex_ptr->v=vc;
						next_vertex_ptr++;
						uc=(uv + *(vil+2))->vx>>1;
						vc=(uv + *(vil+2))->vy>>1;
						next_vertex_ptr->u=uc;
						next_vertex_ptr->v=vc;
						next_vertex_ptr++;

						VIEWPORT_add_prim(vp, next_face_ptr++, ot_pos);
					}
				}
				nf--;
				vil+=TRI;
			}
		}
		mdl=mdl->next_model;
	}
}


void main_game_loop()
{
	int		t1, t2, no_of_frames, tmp, time_diff;
	MATRIX	rotlgt;
	SVECTOR	lgtang={0,2048,0};
	SVECTOR	lgtang2={0,2048,0};
	VECTOR	trans={0,0,0};
	int		watcom_key=0, key;

	/* Set constant color matrix */
	SetColorMatrix(&cmat);

	object_trans.vx=0;
	object_trans.vy=0;
	object_trans.vz=512;

	/* Set color to depth queue */
	SetFarColor(0,0,0);

	do
	{
		/* Rotate light */
		if (light_rotation.vx>0)
			lgtang.vx+=32*no_of_frames;
		if (light_rotation.vy>0)
			lgtang.vy+=32*no_of_frames;
		if (light_rotation.vz>0)
			lgtang.vz+=32*no_of_frames;
		lgtang.vx&=4095;
		lgtang.vy&=4095;
		lgtang.vz&=4095;

		/* Rotate ambient light */
		SetBackColor(shade,shade,shade);

		/* Create constant light matrix */
		RotMatrixYXZ(&lgtang, &light);
		TransMatrix(&light, &trans);
		SetLightMatrix(&light);

		/* keyboard checks */
		if (!keybuf[KEY_CTRL])
		{
			if (keybuf[KEY_Z])
			{
				plot_z=-plot_z;
				vwait(20);
			}
			if (keybuf[KEY_C])
			{
				MDL_centre_object(model1);
			}
			if (keybuf[KEY_V])
			{
				DEVICE_vertical_sync_on(&device);
			}
			if (keybuf[KEY_F3])
			{
				object_angle.vx+=16*no_of_frames;
			}
			if (keybuf[KEY_F4])
			{
				object_angle.vx-=16*no_of_frames;
			}
			if (keybuf[KEY_F5])
			{
				object_angle.vy+=16*no_of_frames;
			}
			if (keybuf[KEY_F6])
			{
				object_angle.vy-=16*no_of_frames;
			}
			if (keybuf[KEY_F7])
			{
				object_angle.vz+=16*no_of_frames;
			}
			if (keybuf[KEY_F8])
			{
				object_angle.vz-=16*no_of_frames;
			}
			if (keybuf[KEY_UP])
			{
				object_trans.vz+=16*no_of_frames;
			}
			if (keybuf[KEY_DOWN])
			{
				object_trans.vz-=8*no_of_frames;
			}
			if (keybuf[KEY_LEFT])
			{
				object_trans.vx+=8*no_of_frames;
			}
			if (keybuf[KEY_RIGHT])
			{
				object_trans.vx-=8*no_of_frames;
			}
			if (keybuf[KEY_F1])
			{
				shade+=64*no_of_frames;
				shade&=(MAX_SHADE-1);
			}
			if (keybuf[KEY_F2])
			{
				shade-=64*no_of_frames;
				shade&=(MAX_SHADE-1);
			}
			if (keybuf[KEY_1])
			{
				light_rotation.vx=-light_rotation.vx;
				vwait(10);
			}
			if (keybuf[KEY_2])
			{
				light_rotation.vy=-light_rotation.vy;
				vwait(10);
			}
			if (keybuf[KEY_3])
			{
				light_rotation.vz=-light_rotation.vz;
				vwait(10);
			}
		}
		else
		{
			if (keybuf[KEY_UP])
			{
				object_trans.vy+=8*no_of_frames;
			}
			if (keybuf[KEY_DOWN])
			{
				object_trans.vy-=8*no_of_frames;
			}
			if (keybuf[KEY_F1])
			{
				poly_type=MWIRE;
				strcpy(type_string, "WIRE FRAME");
			}
			if (keybuf[KEY_F2])
			{
				poly_type=MSOLID;
				strcpy(type_string, "FLAT");
			}
			if (keybuf[KEY_F3])
			{
				strcpy(type_string, "SEMI TRANSPARENT FLAT");
				poly_type=MSOLID|MSEMI;
			}
			if (keybuf[KEY_F4])
			{
				strcpy(type_string, "LIT TEXTURED (NO PERSP)");
				poly_type=MTEXTURED;
			}
			if (keybuf[KEY_F5])
			{
				strcpy(type_string, "SEMI TRANSPARENT LIT TEXTURED (NO PERSP)");
				poly_type=MTEXTURED|MSEMI;
			}
			if (keybuf[KEY_F6])
			{
				strcpy(type_string, "UNLIT TEXTURED (NO PERSP)");
				poly_type=MUNLIT_TEXTURED;
			}
			if (keybuf[KEY_F7])
			{
				strcpy(type_string, "SEMI TRANSPARENT UNLIT TEXTURED (NO PERSP)");
				poly_type=MUNLIT_TEXTURED|MSEMI;
			}
			if (keybuf[KEY_F8])
			{
				strcpy(type_string, "GOURAUD TEXTURED (NO PERSP)");
				poly_type=MTEXTURED|MGOURAUD;
			}
			if (keybuf[KEY_F9])
			{
				strcpy(type_string, "LIT TEXTURED RGB");
				poly_type=MTEXTURED|MRGB;
			}
			if (keybuf[KEY_F10])
			{
				strcpy(type_string, "LIT GOURAUD TEXTURED RGB");
				poly_type=MTEXTURED|MRGB|MGOURAUD;
			}
		}

		/* Masks objects rotation */
		object_angle.vx&=4095;
		object_angle.vy&=4095;
		object_angle.vz&=4095;

		TSC_read(&tmp, &t1);	/* Read time stamp counter */
		SYS_reset_zeus();

		/* Create rotation and translation matrix for object */
		RotMatrix(&object_angle, &object_matrix);
		TransMatrix(&object_matrix, &object_trans);
		SetTransMatrix(&object_matrix);
		SetRotMatrix(&object_matrix);

		if (plot_z>0)
			MDL_plot_model_z(&view, &model1);
		else
			MDL_plot_model(&view, &model1, sxy, sz, poly_type);

		/* Process the device and all viewports etc.. connected to it */
		DEVICE_process();

		TSC_read(&tmp, &t2);	/* Read time stamp counter again */

		/* Calculate frame rate */
		time_diff=cpu_clock_speed/(t2-t1);
		no_of_frames=refresh_rate/time_diff;
		if (no_of_frames==0) no_of_frames++;
		_settextposition(1,1);
		printf("%d\n",time_diff);
		printf("%s\n",type_string);
		frame_count++;
	}
	while (!keybuf[KEY_ESC]);
}

void cleanup_all()
{
	/* De-alloacate buffers */
	SYS_kill_texture_page_mem(tpage0);
	SYS_kill_clut_mem(ramp_clut);
	SYS_kill_clut_mem(rgb_clut);
	SYS_kill_tris(next_index_ptr, next_face_ptr, next_vertex_ptr);
	
	/* Get rid of model and all its materials */
	MDL_kill(&model1[0]);

	/* Close down device and all attached to it */
	DEVICE_delete(&device);
}
void main(int argc, char *argv[])
{

	int			key;
	int			t;
	int			err;
	_RECT		view_dim;
	int			time;
	int			sx=0, sy=0;
	float		scale;

	if (argc<3)
	{
		printf("main file.3ds scale        -     eg main dog.3ds 10.0\n");
		exit(1);
	}
	scale=atof(argv[2]);

	frame_count=0;

	if (SYS_init_zeus()==NULL)
	{
		printf("Could not initialize ZEUS\n");
		exit(1);
	}
	tpage0=SYS_request_texture_page_mem(16*256);
	if (tpage0==NULL)
	{
		printf("Not enough memory for texture page\n");
		exit(1);
	}

	ramp_clut=SYS_request_clut_mem();
	if (ramp_clut==NULL)
	{
		printf("Not enough memory for ramp clut\n");
		exit(1);
	}
	
	rgb_clut=SYS_request_clut_mem();
	if (rgb_clut==NULL)
	{
		printf("Not enough memory for rgb clut\n");
		exit(1);
	}
	
	if (SYS_request_n_tris(&next_index_ptr, &next_face_ptr, &next_vertex_ptr, 20000)==NULL)
	{
		printf("Not enough memory for triangles\n");
		exit(1);
	}

	/* Create primary device */
	DEVICE_clear(&device);
/*	err=DEVICE_create(&device, 320, 200, 8, DEVICE_HWR_LINEAR, 0, VIDEO_RAM_BASE_PC, DEVICE_AUTO_CLEAR, NULL, 1);*/
/*	if (err!=DEVICE_OK)*/
/*	{*/
		err=DEVICE_create(&device, 320, 200, 8, DEVICE_HWR_MCGA_MODE, 0, VIDEO_RAM_BASE_PC, DEVICE_AUTO_CLEAR, NULL, 1);
		if (err!=DEVICE_OK)
		{
			printf("DEVICE ERROR:%d\n",err);
			exit(1);
		}
/*	}*/
	DEVICE_vertical_sync_off(&device);

	DEVICE_load_palette(&device, "../newpal.pal");
	DEVICE_set_palette(&device);

	/* Create us a viewport */
	VIEWPORT_clear(&view);
	view_dim.x=0;
	view_dim.y=0;
	view_dim.w=320;
	view_dim.h=200;
	err=VIEWPORT_create(&view, &view_dim, 0, FP7, 1024, DEVICE_NO_AUTO_CLEAR);
	if (err<0)
	{
		printf("VIEWPORT ERROR:%d\n",err);
	}
	VIEWPORT_set_border(&view, 15);			/* Tells viewport to display a border */
	VIEWPORT_set_min_size(&view, 32,8);		/* Minimized size */
	VIEWPORT_set_max_size(&view, 320,200);	/* Maximized size */
	VIEWPORT_set_near_z(&view, 2);			/* Font Z clipping plane value */
	VIEWPORT_set_interpolation(&view, 4);	/* interpolation during perspective correct texture mapped rendering */
	VIEWPORT_set_perspective(&view, 8);		/* Set eye distance from screen */

	/* Assign various parameters to the viewport */
	VIEWPORT_assign_tpage(&view, tpage0);
	VIEWPORT_assign_ramp_clut(&view, ramp_clut);
	VIEWPORT_assign_rgb_clut(&view, rgb_clut);
	VIEWPORT_assign_semi_clut(&view, semi_clut);
	VIEWPORT_assign_prims(&view, &next_index_ptr, &next_face_ptr, &next_vertex_ptr, &next_vertex_no);


	/* Add the viewport to device */
	DEVICE_add_viewport(&device, &view);
				 
	/* Make device the current device */
	if (err=DEVICE_make_current(&device)<0)
	{
		printf("DEVICE ERROR:%d\n",err);
		exit(1);
	}

	/* Always call at least once before doing anything with it */
	DEVICE_process();


	/* Load ramp and rgb cluts */
	FILE_load_clut("../pal_ref1.tab", ramp_clut);
	FILE_load_clut("../pal_ref2.tab", rgb_clut);

	strcpy(type_string, "LIT TEXTURED (NO PERSP)");
	
	MDL_face_count=0;
	MDL_vert_count=0;

	/* Set scale of 3DS models to be loaded */
	MDL_set_3ds_scale(scale, scale, scale);

	/* Load 3DS model file */
	err=MDL_load_3ds_all(argv[1], model1, &device, 10, tpage0, 16*256, MDL_LINK_OBJECTS);
	if (err!=MDL_OK)
	{
		printf("ERROR loading 3ds %d\n", err);
	}

	/* Recursively center objects */
	for (t=0; t<128; t++)
		MDL_centre_object(model1);

	/* Calculate models normals */
	MDL_calculate_normals(&model1);

	/* Install zeus keyboard handler */
	KBD_clear();
	KBD_install();

	/* call main game loop */
	main_game_loop();


	/* Kill off zeus keyboard handler */
	KBD_kill();

	/* Clean up system */
	cleanup_all();
}

