/**************************************************************
   This example shows how to set up a basic device and viewport
   ,set up primitives and clut and render a basic cube.
**************************************************************/

#include "..\..\..\h\z_all.h"

#define		NAME	"Mr DD Demo 1"

#define		SCREEN_RES_X	320
#define		SCREEN_RES_Y	200


int			app_active;
int			installed=-1;

/* object variables */
	VECTOR			object_trans;
	SVECTOR			object_angle;
	MATRIX			object_matrix;


/* Cube data */
	/* vertices positions relative to cubes origiin */
	SVECTOR			cube_verts[8]={{-128,-128,-128},
									{128,-128,-128},
									{-128,-128,128},
									{128,-128,128},
									{-128,128,-128},
									{128,128,-128},
									{-128,128,128},
									{128,128,128}
									};

	/* Face indices into vertex list (cube_verts) 4 per face */
	short			cube_indices[6*4]={	0,1,3,2,	/* TOP */
										4,5,7,6,	/* BOTTOM */
										2,3,7,6,	/* FRONT */
										1,0,4,5,	/* BACK */
										0,2,6,4,	/* LEFT */
										3,1,5,7		/* RIGHT */
										};

	/* Color of each face (indices into palette) */
	u_short			cube_colors[6]={8,100,120,50,90,200};

/* Device and viewport */
	DEVICE_TYPE		device;
	VIEWPORT_TYPE	view;

	char			*ramp_clut;
	VERTEX			*next_vertex_ptr;
	POLYGON			*next_face_ptr;
	u_short			*next_index_ptr;
	u_short			next_vertex_no;


void draw_cube(SVECTOR *v, short *i, u_short *colors, int nf)
{
	int		t;
	SVECTOR	*v1, *v2, *v3, *v4;
	short	index;
	int		cull;
	long	ot_pos;
	long	p, flag;

	/* Do all faces */
	for (t=0; t<nf; t++)
	{
		/* point to vertices */
		index=*i;
		v1=v + index;
		index=*(i+1);
		v2=v + index;
		index=*(i+2);
		v3=v + index;
		index=*(i+3);
		v4=v + index;

		/* Apply transformation to vertices (SONY PLAYSTATION style) */
		cull=RotAverageNclip4(v1, v2, v3, v4, (long *)next_vertex_ptr,
			(long *)(next_vertex_ptr+1), (long *)(next_vertex_ptr+2),
			(long *)(next_vertex_ptr+3), &p, &ot_pos, &flag);

		/* Only proceed if face is not facing away from us */
		if (cull<0)
		{
			/* tag is always set to NULL for order table purposes */
			next_face_ptr->tag=NULL;

			/* setup faces color and shade value */
			next_face_ptr->c=(*colors);
			next_face_ptr->r=MAX_SHADE>>1;
			next_face_ptr->g=MAX_SHADE>>1;	/* only needed in rgb model */
			next_face_ptr->b=MAX_SHADE>>1;	/* only needed in rgb model */

			/* setup number of vertices */
			next_face_ptr->nv=QUAD;

			/* setup primitive vertices */
			next_face_ptr->v=next_vertex_ptr;

			/* setup face index table */
			next_face_ptr->face_data=next_index_ptr;

			/* setup indices to primitive vertices */
			*(next_index_ptr++)=0;
			*(next_index_ptr++)=1;
			*(next_index_ptr++)=2;
			*(next_index_ptr++)=3;

			/* setup polygon type */
			next_face_ptr->id=MSOLID|MRAMP;

			next_face_ptr->rgb=0;
			next_face_ptr->tpage=0;
			next_face_ptr->semi=0;

			/* Add primitive to viewport */
			/* Zeus does support AddPrim() but you must get the order table
			   address from the viewport ie view.ot and check for a primitive
			   going out of range (view.ot_size - size of order table) */
			VIEWPORT_add_prim(&view, next_face_ptr++, ot_pos);

			/* Move to next available set of vertices */
			next_vertex_ptr+=QUAD;
		}

		/* Move to next set of source indices */
		i+=QUAD;

		/* Move to next face color */
		colors++;
	}
}


/*************************************************/
/* Main loop responsible for rotating models etc */
/*************************************************/
BOOL main_loop()
{
	char	err_str[16];
	int		err;

	object_angle.vy+=32;
	/* Masks objects rotation */
	object_angle.vx&=4095;
	object_angle.vy&=4095;
	object_angle.vz&=4095;

	/* Create rotation and translation matrix for object */
	RotMatrix(&object_angle, &object_matrix);
	TransMatrix(&object_matrix, &object_trans);

	/* Set as constant rotation and translation matrix */
	SetTransMatrix(&object_matrix);
	SetRotMatrix(&object_matrix);

	/* Draw the object */
	draw_cube(&cube_verts[0], &cube_indices[0], &cube_colors[0], 6);

	/* Process the device and all viewports etc.. connected to it */
	err=DEVICE_process();
	if (err!=DEVICE_OK)
	{
		printf("DEVICE ERROR %d\n", err);

		return(FALSE);
	}
	frame_count++;

	return(TRUE);
}

BOOL setup_all()
{
	int					err;
	char				err_str[16];
	_RECT				view_dim;
	int					t;

	/* This is just so I can keep track of number of game frames */
	frame_count=0;

	/* Initialize zeus */
	if (SYS_init_zeus()==NULL)
	{
		printf("Could not initialize zeus\n");
		return(FALSE);
	}

	/* Request a clut (FLAT color model) */
	ramp_clut=SYS_request_clut_mem();
	if (ramp_clut==NULL)
	{
		printf("Not enough memory for ramp clut\n");
		return(FALSE);
	}
	
	/* Request 20,000 graphics primitives */
	if (SYS_request_n_quads((int *)&next_index_ptr, (int *)&next_face_ptr, (int *)&next_vertex_ptr, 20000)==NULL)
	{
		printf("Not enough memory for quadrangles\n");
		return(FALSE);
	}

	/* Clear device structure */
	DEVICE_clear(&device);

	/* Create primary device */
	err=DEVICE_create(&device, SCREEN_RES_X, SCREEN_RES_Y, 8, NULL,
			0, VIDEO_RAM_BASE_PC, DEVICE_AUTO_CLEAR, NULL, 1);
	if (err!=DEVICE_OK)
	{
		printf("DEVICE ERROR %d\n"), err;

		return(FALSE);
	}

	/* Turn vertical sync off */
	DEVICE_vertical_sync_off(&device);

	/* Load palette and set */
	DEVICE_load_palette(&device, "../newpal.pal");
	DEVICE_set_palette(&device);

	/* Create us a viewport */
	VIEWPORT_clear(&view);
	view_dim.x=0;
	view_dim.y=0;
	view_dim.w=SCREEN_RES_X-1;
	view_dim.h=SCREEN_RES_Y-1;
	err=VIEWPORT_create(&view, &view_dim, 0, FP7, 1024, DEVICE_NO_AUTO_CLEAR);
	if (err!=VIEWPORT_OK)
	{
		printf("VIEWPORT ERROR %d\n", err);

		return(FALSE);
	}
	VIEWPORT_set_border(&view, 15);			/* Tells viewport to display a border */
	VIEWPORT_set_min_size(&view, 32,8);		/* Minimized size */
	VIEWPORT_set_max_size(&view, 320,200);	/* Maximized size */
	VIEWPORT_set_near_z(&view, 0);			/* Font Z clipping plane value */
	VIEWPORT_set_interpolation(&view, 4);	/* interpolation during perspective correct texture mapped rendering */
	VIEWPORT_set_perspective(&view, 8);		/* Set eye distance from screen */

	/* Assign clut to viewport */
	VIEWPORT_assign_ramp_clut(&view, ramp_clut);

	/* Assign primitives to viewport */
	VIEWPORT_assign_prims(&view, (int *)&next_index_ptr,
		(int *)&next_face_ptr, (int *)&next_vertex_ptr, (int *)&next_vertex_no);

	/* Add the viewport to device */
	DEVICE_add_viewport(&device, &view);
				 
	/* Make 'device' the current working device */
	if (err=DEVICE_make_current(&device)<0)
	{
		printf("COULD NOT install devive \n");

		return(FALSE);
	}

	/* Make 'view' current working viewport */
	VIEWPORT_make_current(&view);

	/* Always call at least once before doing anything with it */
	DEVICE_process();

	/* Load ramp clut */
	FILE_load_clut("../pal_ref1.tab", ramp_clut);

	/* Set objects position in space */
	object_trans.vx=0;
	object_trans.vy=0;
	object_trans.vz=512;

	return(TRUE);
}
void cleanup_all()
{
	/* Close down device, all viewports, order tables, primitive data, palettes etc.. */
	DEVICE_delete(&device);
}

void main()
{
	/* Make a device, viewport and load in graphics files etc */
	if (!(setup_all()))
	    return;

	installed=TRUE;

	/* Install keyboard handler */
	KBD_install();

	/* MAIN LOOP CALL */
	do
	{ 
		if (!main_loop())
			return;
	}
	while(!keybuf[KEY_ESC]);	/* continue until escape key pressed */

	/* kill keyboard handler */
	KBD_kill();

	cleanup_all();

	set_video_mode(3);			/* set text video mode */
}
