// GraphX library
// Copyright (c) 1996, Ben Crowder

#include "graphx.h"
unsigned char far* video_buffer = (char far*) 0xA0000000L;

// SET MODE FUNCTIONS
void SetVGA();
void SetTXT();
void SetMode(int mode);

// GRAPHICS PRIMITIVES
void PutPixel(int x, int y, int color);
int GetPixel(int x, int y);
void HorizLine(int x, int x2, int y, int color);
void VertLine(int x, int y, int y2, int color);

// PALETTE FUNCTIONS
void SetPal(int colornum, int red, int green, int blue);
void GetPal(int colornum, UCHAR* red, UCHAR* green, UCHAR* blue);
void RotatePal(int start, int end, int num, int dir);
void GrabPal(unsigned char pal[]);
void BlackOutPal();
void FadeOut();
void FadeIn(unsigned char pal[]);

// SPRITE FUNCTIONS
void PutBitmap(int bitmap[], int x, int y);
void PutBitmapT(int bitmap[], int x, int y);
void CutOut(int bitmap[], int x, int y);
void SaveBitmap(int bitmap[], char* filename);
void LoadBitmap(int* bitmap[], char* filename);
void CopyBitmap(int source[], int* dest[]);

// MISCELLANEOUS FUNCTIONS
void WaitRetrace();

// KEYBOARD FUNCTIONS
int CheckKey(int key);

// MOUSE FUNCTIONS
void InitMouse();
void ShowMouse();
void HideMouse();
int GetMouseX();
int GetMouseY();
int TXTGetMouseX();
int TXTGetMouseY();
int MGetStat();

// PCX FUNCTIONS
void LoadPCX(char *filename, struct PCX_Picture *image, int setpal);
void ShowPCX(struct PCX_Picture image, int x, int y);
void SetPCXPal(struct PCX_Picture image);
void FreePCX(struct PCX_Picture *pcx);

// SPECIAL EFFECTS
void BlackOutDot();
void BlackOutLine(int delayms, int type);
void BlackSweep(int direction, int type, int delayms);

// MATH FUNCTIONS
long Factorial(long n);
long Power(long n, long exp);

void main()
{
	int x=160,y=100,i;
	PCX_Picture box;
	int cursor[] = {8,8,
			0, 0,14,14,14,14, 0, 0,
			0,14, 0, 0, 0, 0,14, 0,
		 14, 0, 4, 0, 0, 4, 0,14,
		 14, 0, 0, 4, 4, 0, 0,14,
		 14, 0, 0, 4, 4, 0, 0,14,
		 14, 0, 4, 0, 0, 4, 0,14,
			0,14, 0, 0, 0, 0,14, 0,
			0, 0,14,14,14,14, 0, 0};
	int mask[]= {9,9,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,0};

	randomize();
	SetVGA();
	LoadPCX("BACKGRND.PCX",&box,1);

	for (i=0;i<10000;i++)
	{
		ShowPCX(box,random(315),random(195));
	}
	CutOut(mask,x,y);
	PutBitmapT(cursor,x,y);

	do
	{
		PutBitmap(mask,x,y);

		if ( (CheckKey(72)) && (y > 0) ) y--;
		if ( (CheckKey(75)) && (x > 0) ) x--;
		if ( (CheckKey(77)) && (x < 311) ) x++;
		if ( (CheckKey(80)) && (y < 191) ) y++;

		CutOut(mask,x,y);
		PutBitmapT(cursor,x,y);

		delay(25);
	} while (!CheckKey(1));
	BlackOutLine(10,2);

	SetTXT();
}

// SET MODE FUNCTIONS ////////////////////////////////////////////////////////

void SetVGA()
{
	asm {
	 mov ax,0x13;
	 int 10h;
	}
}

void SetTXT()
{
	asm {
	 mov ax,0x03;
	 int 10h;
	}
}

void SetMode(int mode)
{
	asm {
	 mov ax,[mode];
	 int 10h;
	}
}

// GRAPHICS PRIMITIVES ///////////////////////////////////////////////////////

void PutPixel(int x, int y, int color)
{
	video_buffer[ ( (y << 8) + (y << 6) ) + x] = color;
}

int GetPixel(int x, int y)
{
	return video_buffer[ ( (y << 8) + (y << 6) ) + x];
}

void HorizLine(int x, int x2, int y, int color)
{
	int i;

	for (i=x;i<=x2;i++)
	{
		PutPixel(i,y,color);
	}
}

void VertLine(int x, int y, int y2, int color)
{
	int i;

	for (i=y;i<=y2;i++)
	{
		PutPixel(x,i,color);
	}
}

// PALETTE FUNCTIONS /////////////////////////////////////////////////////////

void SetPal(int colornum, int red, int green, int blue)
{
	outp(0x3C8,colornum);
	outp(0x3C9,red);
	outp(0x3C9,green);
	outp(0x3C9,blue);
}

void GetPal(int colornum, UCHAR* red, UCHAR* green, UCHAR* blue)
{
	outp(0x3C7,colornum);
	*red = inp(0x3C9);
	*green = inp(0x3C9);
	*blue = inp(0x3C9);
}

void RotatePal(int start, int end, int num, int dir)
{
	int i,j;
	UCHAR r[256],g[256],b[256];

	for (i=0;i<256;i++)
	{
		GetPal(i,&r[i],&g[i],&b[i]);
	}

	for (i=0;i<num;i++)
	{
		for (j=start;j<end;j++)
		{
			switch(dir) {
			 case 0 :
				r[j] = r[j]+1;
				g[j] = g[j]+1;
				b[j] = b[j]+1;
				break;
			 case 1 :
				r[j] = r[j]-1;
				g[j] = g[j]-1;
				b[j] = b[j]-1;
				break;
			}
			SetPal(j,r[j],g[j],b[j]);
		}
	}
}

void GrabPal(unsigned char pal[])
{
	int loop;

	for(loop=0;loop<255;loop++) {
		GetPal(loop,&pal[loop],&pal[loop+256],&pal[loop+256]);
	}
}

void BlackOutPal()
{
	int loop;

	WaitRetrace();

	for(loop=0;loop<255;loop++) {
		SetPal(loop,0,0,0);
	}
}

void FadeOut()
{
	int loop1,loop2;
	unsigned char pal[3];

	for(loop1=1;loop1<64;loop1++) {
		WaitRetrace();
		for(loop2=1;loop2<255;loop2++) {
			GetPal(loop2,&pal[0],&pal[1],&pal[2]);
			if (pal[0]>0) pal[0]--;
			if (pal[1]>0) pal[1]--;
			if (pal[2]>0) pal[2]--;
			SetPal(loop2,pal[0],pal[1],pal[2]);
		}
	}
}

void FadeIn(unsigned char Pal[])
{
	int loop1,loop2;
	UCHAR pal[3];

	for(loop1=1;loop1<64;loop1++) {
		WaitRetrace();
		for(loop2=0;loop2<255;loop2++) {
			GetPal(Pal[loop2],&pal[0],&pal[1],&pal[2]);
			if (pal[0] < pal[0]) pal[0]++;
			if (pal[1] < pal[1]) pal[1]++;
			if (pal[2] < pal[2]) pal[2]++;
			SetPal(Pal[loop2],pal[0],pal[1],pal[2]);
		}
	}
}

// SPRITE FUNCTIONS //////////////////////////////////////////////////////////

void PutBitmap(int bitmap[], int x, int y)
{
	int width=bitmap[0];
	int height=bitmap[1];
	int i,j;
	int cnt=2;

	for (i=y;i<y+height;i++) {
		for (j=x;j<x+width;j++) {
			PutPixel(j,i,bitmap[cnt]);
			cnt++;
		}
	}
}

void PutBitmapT(int bitmap[], int x, int y)
{
	int width=bitmap[0];
	int height=bitmap[1];
	int i,j;
	int cnt=2;

	for (i=y;i<y+height;i++) {
		for (j=x;j<x+width;j++) {
			if (bitmap[cnt] != 0) PutPixel(j,i,bitmap[cnt]);
			cnt++;
		}
	}
}

void CutOut(int bitmap[], int x, int y)
{
	int width=bitmap[0];
	int height=bitmap[1];
	int i,j;
	int cnt=2;

	for (i=y;i<y+height;i++) {
		for (j=x;j<x+width;j++) {
			bitmap[cnt]=GetPixel(j,i);
			cnt++;
		}
	}
}

void SaveBitmap(int bitmap[], char* filename)
{
	ofstream output;

	output.open(filename,ios::binary);
	output.write( (char*)&bitmap, sizeof(bitmap) );
	output.close();
}

void LoadBitmap(int* bitmap[], char* filename)
{
	ifstream input;

	input.open(filename,ios::binary);
	input.read( (char*)&bitmap, sizeof(bitmap) );
	input.close();
}

void CopyBitmap(int source[], int* dest[])
{
	int width,height,i;

	width = source[0];
	height = source[1];

	for (i=2;i<width+height+2;i++)
	{
		*dest[i] = source[i];
	}
}

// MISC. FUNCTIONS ///////////////////////////////////////////////////////////

void WaitRetrace()
{
	l1:; l2:;
	asm {
		mov dx,0x03DA;
  l1:
		in al,dx;
		and al,0x08;
		jnz l1;
	l2:
		in al,dx
		and al,0x08;
		jz l2;
	}
}

// KEYBOARD FUNCTIONS ////////////////////////////////////////////////////////

int CheckKey(int key)
{
	int i;
	i = inp(0x60);

	if (i == key) return 1;
	return 0;
}

// MOUSE FUNCTIONS ///////////////////////////////////////////////////////////

void InitMouse()
{
	asm {
		mov ax, 0x00;
		int 33h;
	}
}

void ShowMouse()
{
	asm {
		mov ax, 0x01;
		int 33h;
	}
}

void HideMouse()
{
	asm {
		mov ax, 0x02;
		int 33h;
	}
}

int GetMouseX()
{
	int x;
	asm {
		mov ax,0x03;
		int 33h;
		mov [x],cx;
	}
	return x;
}

int GetMouseY()
{
	int y;
	asm {
		mov ax,0x03;
		int 33h;
		mov [y],dx;
	}
	return y;
}


int TXTGetMouseX()
{
	int x;
	asm {
		mov ax,0x03;
		int 33h;
		mov [x],cx;
	}
	return x / 8;
}

int TXTGetMouseY()
{
	int y;
	asm {
		mov ax,0x03;
		int 33h;
		mov [y],dx;
	}
	return y / 8;
}

int MGetStat()
{
	int butstat;
	asm {
		mov ax,0x03;
		int 33h;
		mov [butstat],bx;
	}
	return butstat;
}

// PCX FUNCTIONS /////////////////////////////////////////////////////////////

void LoadPCX(char *filename, struct PCX_Picture *image, int setpal)
{
	FILE *fp;
	unsigned char *temp = (char *) &(image->header);
	long count;
	unsigned char data, runlen;

	if((fp = fopen(filename, "rb")) == NULL)
	{
		printf("Error opening file: %s.\n", filename);
		exit(1);
	}

	if(!fread(temp, 1, 128, fp))
	{
		printf("Error reading PCX header information.\n");
		exit(1);
	}

	image->header.width++;
	image->header.height++;

	if((image->buffer = (unsigned char far *) farmalloc((long) image->header.width * image->header.height)) == NULL)
	{
		printf("Error allocating memory.\n");
		exit(1);
	}

	count = 0;
	while(count < (long) image->header.width * image->header.height)
	{
		if(!fread(&data, 1, 1, fp))
		{
			printf("Error reading PCX data.\n");
			exit(1);
		}
		if(data >= 192)
		{
			runlen = data - 192;
			if(!fread(&data, 1, 1, fp))
			{
				printf("Error reading PCX data.\n");
				exit(1);
			}
			while(runlen-->0)
			{
				image->buffer[count++] = data;
			}
		}
		else
		{
			image->buffer[count++] = data;
		}
	}

	fseek(fp, -768L,SEEK_END);

	for(count = 0; count < 256; count++)
	{
		if(!fread(&data, 1, 1, fp))
		{
			printf("Error reading PCX pallete.\n");
			exit(1);
		}
		image->r[count] = data >> 2;
		if(!fread(&data, 1, 1, fp))
		{
			printf("Error reading PCX data.\n");
			exit(1);
		}
		image->g[count] = data >> 2;
		if(!fread(&data, 1, 1, fp))
		{
			printf("Error reading PCX data.\n");
			exit(1);
		}
		image->b[count] = data >> 2;
		if(setpal) SetPal(count, image->r[count], image->g[count], image->b[count]);
	}
  fclose(fp);
}

void ShowPCX(struct PCX_Picture image, int x, int y)
{
	int i, j;
	unsigned char far *src;
	unsigned char far *dest;

	src = image.buffer;
	dest = video_buffer + x + (y << 6) + (y << 8);

	for(i = 0; i < image.header.height; i++)
	{
		for(j = 0; j < image.header.width; j++)
		{
			dest[j] = src[j];
		}
		src += image.header.width;
		dest += 320;
	}
}

void SetPCXPal(struct PCX_Picture image)
{
	int count;

	for(count = 0; count < 256; count++)
		SetPal(count, image.r[count], image.g[count], image.b[count]);
}

void FreePCX(struct PCX_Picture *pcx)
{
	free((void *) pcx->buffer);
}

// SPECIAL EFFECTS

void BlackOutDot()
{
	unsigned long i;
	for (i=1;i<=300000;i++) {
		PutPixel(random(320),random(200),0);
	}
}

void BlackOutLine(int delayms, int type)
{
	int i,j;

	switch(type)
	{
		case 0 :
		 for (i=0;i<=200;i+=2)
			{
				HorizLine(0,320,i,0);
				delay(delayms);
			}
		 for (i=1;i<=200;i+=2)
			{
				HorizLine(0,320,i,0);
				delay(delayms);
			}
		 break;
		case 1 :
		 for (i=0;i<=320;i+=2)
			{
				VertLine(i,0,200,0);
				delay(delayms);
			}
		 for (i=1;i<=320;i+=2)
			{
				VertLine(i,0,200,0);
				delay(delayms);
			}
		 break;
	}
}

// Direction 0 = left, 1 = right, or if type >= 2 then 0 = in, 1 = out
// Type 0 = horiz, 1 = vert, 2 = both horiz, 3 = both vert, 4 = all
void BlackSweep(int direction, int type, int delayms)
{
	int i,j,k,l;

	switch(type)
	{
		case 0 :
			if (direction == 0) {
				for (i=199;i>=0;i--)
				{
					HorizLine(0,320,i,0);
					delay(delayms);
				}
			}
			if (direction == 1) {
				for (i=0;i<=199;i++)
				{
					HorizLine(0,320,i,0);
					delay(delayms);
				}
			}
			break;
		case 1 :
			if (direction == 0) {
				for (i=319;i>=0;i--)
				{
					VertLine(i,0,200,0);
					delay(delayms);
				}
			}
			if (direction == 1) {
				for (i=0;i<=319;i++)
				{
					VertLine(i,0,200,0);
					delay(delayms);
				}
			}
			break;
		case 2 :
			if (direction == 0)
			{
				for (i=199,j=0;i>=0,j<=199;i--,j++)
				{
					HorizLine(0,320,i,0);
					HorizLine(0,320,j,0);
					delay(delayms);
				}
			}
			if (direction == 1)
			{
				for (i=100,j=101;i>=0,j<=199;i--,j++)
				{
					HorizLine(0,320,i,0);
					HorizLine(0,320,j,0);
					delay(delayms);
				}
			}
		 break;
		case 3 :
			if (direction == 0)
			{
				for (i=319,j=0;i>=0,j<=319;i--,j++)
				{
					VertLine(i,0,200,0);
					VertLine(j,0,200,0);
					delay(delayms);
				}
			}
			if (direction == 1)
			{
				for (i=160,j=161;i>=0,j<=319;i--,j++)
				{
					VertLine(i,0,200,0);
					VertLine(j,0,200,0);
					delay(delayms);
				}
			}
		 break;
		case 4 :
			if (direction == 0)
			{
				for (i=319,j=0,k=199,l=0;i>=0,j<=319,k>=0,l<=199;i--,j++,k--,l++)
				{
					VertLine(i,0,200,0);
					VertLine(j,0,200,0);
					HorizLine(0,320,k,0);
					HorizLine(0,320,l,0);
					delay(delayms);
				}
			}
			if (direction == 1)
			{
				for (i=160,j=161,k=100,l=101;i>=0,j<=319,k>=0,l<=199;i--,j++,k--,l++)
				{
					VertLine(i,0,200,0);
					VertLine(j,0,200,0);
					HorizLine(0,320,k,0);
					HorizLine(0,320,l,0);
					delay(delayms);
				}
			}
		 break;
	}
}

// MATH FUNCTIONS ////////////////////////////////////////////////////////////

long Factorial(long n)
{
	long fact;
	if (n > 0)
	{
		fact = n * Factorial(n-1);
	}
	else
	{
		fact = 1;
	}
	return fact;
}

long Power(long n, long exp)
{
	int i;
	long temp=n;

	for (i=0;i<exp;i++)
	{
		temp *= n;
	}
	return temp;
}