/*_________________________________________________________________________*\
                            \/ /            \ \/
                              <  FM Example  >
                               \____________/

                          VARMINT'S AUDIO TOOLS 1.1


  This file is part of the VAT 1.1 distribution set. It is being provided
  as part of the VAT 1.1 demo distribution as a courtesy to prospective
  Ground Up customers. Unregistered users are granted a limited license to
  compile and run this code for demonstration purposes.  Please see the
  LICENSE.TXT file for information on how you may use the VAT library.

  This program demonstrates the VAT FM API (Application Programming
  Interface).  Please refer to your VAT Reference Manual for information
  not covered here.  Correspondence should be directed to:

    support@groundup.com

  We very much encourage your comments and suggestions.  We want VAT to
  be useful and enjoyable for you.

    Please visit our web site at:

        http://www.groundup.com/

                                            Copyright 1995,1996 Ground Up
\*_________________________________________________________________________*/


#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "vat.h"                  // VAT's header file, always mandatory

#define DOWN 0                    // some definitions to clarify the examples
#define UP   1

void showmodes (int modes);       // function for rhythm demo


//-------------------------------- main () --------------------------------//

int main ()
{
  INSTRUMENT *inst; // INSTRUMENT array, used to capture default set
  int   inst_indx;  // instrument index
  int   note;       // used for note setting
  int   quit;       // quit flag used in rhythm demo
  int   voice;      // used for FM channel setting
  int   modes;      // used for temporary rhythm mode retrieval
  int   m;          // used for even more temporary bitfield calculation
  int   dir;        // used in rhythm demo for note changes
  char  c;          // used to hold input characters


  printf ("Initializing SoundBlaster at 45kHz.\n");
    //
    // Start VAT at 45Khz. As we are in a text mode the refresh setting
    // is not important; we'll use 70hz for it. If there is a problem,
    // SBSetUp() returns 0 and the sberr global variable is set to an
    // index into the errname global variable which can be used to report
    // the cause of the error.
    //
  if (!SBSetUp (45454, 70)) {
    printf ("Error in SBSetUp(): %s\n", errname[sberr]);
    exit (1);
  }
    //
    // SBSetUp() must be paired with GoVarmint() before any audio output.
    //
  GoVarmint ();
    //
    // Report the actual rate VAT was initialized at. This may be altered
    // from the rate specified above in two ways:
    //  1. The rate will be slightly adjusted to one of many predefined
    //      levels which were found to be optimal for audio output.
    //  2. If the VAT library used is the shareware or demo version, the
    //      rate will be limited to 11kHz.
    //
  printf ("Actually initialized to: %dHz\n", SampleRate ());


  printf ("Getting default instrument set...\n");
    //
    // Retrieve the instrument set in use by VAT using MidiInstruments()
    // with the special GET_INSTRUMENTS definition. Because we have not
    // activated a different instrument set, the default instrument set
    // will be returned.
    //
  inst = MidiInstruments (GET_INSTRUMENTS);


    //
    // Play a sample of each of the default instruments.
    //
  printf ("Playing six C's for every instrument... (hit a key to stop)\n");
  inst_indx = 0;    // initialize the instrument index to zero
  note = 36;        // initialize the note to 36, the second 'C' note
  FMVolume (0, 63); // initialize the first FM channel's volume to maximum

    //
    // Loop through all 128 default instruments, playing notes 36, 48, 60,
    // 72, 84, and 96 (increments of 12 - an octave). The loop will exit
    // either when all 128 instruments have been sampled or a key is pressed.
    //
  while (!kbhit () && inst_indx < 128) {
      //
      // If the note is now greater than 96, it's time to move on to the
      // next instrument. Reset the note to 36 and increment inst_indx.
      //
    if (note > 96) {
      note = 36;
      inst_indx++;
    }
      //
      // If the note is now 36, this is a new instrument. Say so and program
      // the first FM channel with the data of the current instrument.
      //
    if (note == 36) {
      printf ("\ninstrument %3d [%s]...", inst_indx, inst[inst_indx].name);
      fflush (stdout);
      FMVoice (0, inst[inst_indx].data);
    }
    FMNote (0, note); // set the first FM channel's pitch to the current note
    FMKeyOn (0);      // 'press' the first FM channel's controller
    MilliDelay (200); // wait a little while (2/10 of a second)
    FMKeyOff (0);     // 'release' the first FM channel's controller
    note += 12;       // increment the note to the next octave
  }
    //
    // If the loop exited because of a keypress, grab it and throw it away.
    //
  if (kbhit ())
    getch ();


    //
    // Display information on how to control the FM rhythm demo.
    //
  printf ("\nPlaying inst in rhythm mode:\n"
          "  q - quit\n"
          "  ? - this listing\n"
          "  1 - toggle 'hihat' rhythm mode\n"
          "  2 - toggle 'topcym' rhythm mode\n"
          "  3 - toggle 'tomtom' rhythm mode\n"
          "  4 - toggle 'snare' rhythm mode\n"
          "  5 - toggle 'bass' rhythm mode\n"
          "  6 - use voice 6\n"
          "  7 - use voice 7\n"
          "  8 - use voice 8\n"
          "  + - go to next instrument\n"
          "  - - go to previous instrument\n");

  inst_indx = 0;  // initialize the instrument index to zero
  modes = 0;      // initialize the rhythm mode bitfield to zero
  voice = 6;      // initialize the FM channel number used to six
  note = 60;      // initialize the note to 60, middle C
  dir = 1;        // initialize the direction to nonzero, meaning up
  quit = 0;       // initialize the quit flag to zero


    //
    // Note the current instrument and rhythm modes in use. This will of
    // course be instrument 0 with no rhythm modes set at this point. Also
    // note the current channel, which is set to 6 above.
    //
  printf ("<instrument %3d [%s]>\n", inst_indx, inst[inst_indx].name);
  showmodes (modes);
  printf ("<using voice bank %d>\n", voice);


    //
    // Start the rhythm demo loop, which will continue to run until the
    // quit flag is set to nonzero, which happens when the user hits the
    // 'q' key.
    //
  while (!quit) {
      //
      // Play the current FM channel with the current instrument at the
      // current pitch to demonstrate the current rhythm mode's effect.
      // Start by programming the current FM channel with the current
      // instrument's data.
      //
    FMVoice (voice, inst[inst_indx].data);
    FMNote (voice, note); // set the current FM channel's pitch
    FMVolume (voice, 63); // set its volume to maximum
    FMKeyOn (voice);      // 'press' its controller
    MilliDelay (100);     // wait a little while (1/10 of a second)
    FMKeyOff (voice);     // 'release' its controller
    FMVolume (voice, 0);  // set its volume to minimum
    if (dir == UP) {
      note += 12;         // increment the note an octave
      if (note + 12 > 72) // until we pass 72
        dir = DOWN;
    } else {
      note -= 12;         // and decrement the note an octave
      if (note - 12 < 36) // until we pass 36
        dir = UP;
    }

    if (!kbhit ())        // check for a keypress using the standard method
      continue;
    c = getch ();        // if a keypress was detected, grab it

      //
      // Scan through the valid command keys.
      //
    switch (c) {
    case 'Q':
    case 'q':
        //
        // Set the quit flag to nonzero, indicating the play loop should
        // be exited.
        //
      quit = 1;
      printf ("<quit>\n");
      break;
    case '?':
        //
        // Display information on how to control the FM rhythm demo.
        //
      printf ("\nPlaying inst in rhythm mode:\n"
              "  q - quit\n"
              "  ? - this listing\n"
              "  1 - toggle 'hihat' rhythm mode\n"
              "  2 - toggle 'topcym' rhythm mode\n"
              "  3 - toggle 'tomtom' rhythm mode\n"
              "  4 - toggle 'snare' rhythm mode\n"
              "  5 - toggle 'bass' rhythm mode\n"
              "  6 - use voice bank 6\n"
              "  7 - use voice bank 7\n"
              "  8 - use voice bank 8\n"
              "  + - go to next instrument\n"
              "  - - go to previous instrument\n");
      break;
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
      m = (1<<(c-'1'));
        //
        // Toggle the indicated rhythm mode with FMRhythm(). This program
        // doesn't need to keep track of the FM rhythm mode setting with
        // the help of the special GET_SETTING definition, available for
        // nearly all value setting functions in VAT.
        //
      modes = FMRhythm (GET_SETTING);  // get the current rhythm mode
      if (modes & m)                   // if the bit is set
        modes &= ~m;                   // clear it
      else                             // otherwise
        modes |= m;                    // set it
      FMRhythm (modes);                // send the new rhythm mode
      showmodes (modes);               // and display the current setting
      break;
    case '6':
    case '7':
    case '8':
        //
        // Set the voice variable to the selection and note the change.
        // This is the FM channel selection command.
        //
      if (voice != (c-'0')) {
        voice = c-'0';
        printf ("<using voice bank %d>\n", voice);
      }
      break;
    case '+':
        //
        // Increment the instrument index, wrapping around if necessary,
        // and note the change.
        //
      if (inst_indx == 127)
        inst_indx = 0;
      else
        inst_indx++;
      printf ("<instrument %3d [%s]>\n", inst_indx, inst[inst_indx].name);
      break;
    case '-':
        //
        // Decrement the instrument index, wrapping around if necessary,
        // and note the change.
        //
      if (!inst_indx)
        inst_indx = 127;
      else
        inst_indx--;
      printf ("<instrument %3d [%s]>\n", inst_indx, inst[inst_indx].name);
      break;
    }
  }


    //
    // Shut down VAT and exit.
    //
  printf ("\nShutting down VAT.\n");
  DropDeadVarmint ();
  SBCleanUp ();
  return 0;
}


//---------------------------- report_chain () ----------------------------//

  //
  // This function is used in the FM rhythm demo. It prints out human
  // readable representations of the values in a rhythm mode bitfield.
  //
void showmodes (int modes)
{
  printf ("<modes: ");
  if (!modes)
    printf ("(none)>\n");
  else {
    if (modes & v_hihat_rhythm)
      printf ("hihat,");
    if (modes & v_topcym_rhythm)
      printf ("topcym,");
    if (modes & v_tomtom_rhythm)
      printf ("tomtom,");
    if (modes & v_snare_rhythm)
      printf ("snare,");
    if (modes & v_bass_rhythm)
      printf ("bass,");
    printf ("\b>\n");
  }
}

