/*_________________________________________________________________________*\
			   \/ /               \ \/
			     <  Mixer Example  >
			      \_______________/

			  VARMINT'S AUDIO TOOLS 1.1


  This file is part of the VAT 1.1 distribution set. It is being provided
  as part of the VAT 1.1 demo distribution as a courtesy to prospective
  Ground Up customers. Unregistered users are granted a limited license to
  compile and run this code for demonstration purposes.  Please see the
  LICENSE.TXT file for information on how you may use the VAT library.

  This program demonstrates the VAT Mixer API (Application Programming
  Interface).  Please refer to your VAT Reference Manual for information
  not covered here.  Correspondence should be directed to:

    support@groundup.com

  We very much encourage your comments and suggestions.  We want VAT to
  be useful and enjoyable for you.

    Please visit our web site at:

	http://www.groundup.com/

					    Copyright 1995,1996 Ground Up
\*_________________________________________________________________________*/


#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include <graph.h>
#include "vat.h"                // VAT's header file, always mandatory

  // macros to simplify part of the example
#define gotoxy(x,y)       _settextposition(y,x)
#define clrscr()          _clearscreen(0)
#define setct(x)          _settextcursor(x)

  // definitions to simplify part of the example
#define SOLIDCURSOR   0x0007
#define NORMALCURSOR  0x0607
#define NOCURSOR      0x2000


//-------------------------------- main () --------------------------------//

int main ()
{
  int   value;                 // temporary setting variable
  int   quit;                  // quit flag used in mixer control loop
  char  c;                     // used to hold input characters
  MIXER_STATE *initial_state;  // holds the initial state of the mixer


  printf ("Initializing SoundBlaster at 4kHz.\n");
    //
    // Start VAT at 4Khz. As we are in a text mode the refresh setting
    // is not important; we'll use 70hz for it. Furthermore, since we're
    // not going to be performing any audio tasks, the sample rate is
    // unimportant here and we don't need to use GoVarmint(). If there is
    // a problem with SBSetUp(), it returns 0 and the sberr global variable
    // is set to an index into the errname global variable which can be
    // used to report the cause of the error.
    //
  if (!SBSetUp (4000, 70)) {
    printf ("Error in SBSetUp(): %s\n", errname[sberr]);
    exit (1);
  }


    //
    // Get the state of the mixer before the user changes anything, using
    // MixerState() with the special GET_STATE definition.
    //
  initial_state = MixerState (GET_STATE);


  setct (NOCURSOR);  // disable the cursor
  clrscr ();         // clear the screen

  quit = 0;  // initialize the quit flag to zero


    //
    // Start the mixer control loop, which will continue to run until the
    // quit flag is set to nonzero, which happens when the user hits the
    // 'q' key.
    //
  while (!quit) {
      //
      // Retrieve and display various mixer values using MixerSetting()
      // with the special GET_SETTING definition.
      //
    gotoxy (0,0);
    printf ("                         Dump of SB Mixer Registers\n\n");

    printf ("                                    /   Value\n");
    printf ("                                    ---   -----\n");
    
    value = MixerSetting (v_mas_volume, GET_SETTING);   // master volume
    printf ("                            Volume (V/v): 0x%0.2X\n", value);

    value = MixerSetting (v_wave_volume, GET_SETTING);  // wave volume
    printf ("                            Voice  (W/w): 0x%0.2X\n", value);

    value = MixerSetting (v_midi_volume, GET_SETTING);  // midi volume
    printf ("                            Midi   (M/m): 0x%0.2X\n", value);

    value = MixerSetting (v_cd_volume, GET_SETTING);    // CD volume
    printf ("                            CD     (C/c): 0x%0.2X\n", value);

    value = MixerSetting (v_line_volume, GET_SETTING);  // line volume
    printf ("                            Line   (L/l): 0x%0.2X\n", value);

    value = MixerSetting (v_mas_treble, GET_SETTING);   // treble
    printf ("                            Treble (T/t): 0x%0.2X\n", value);

    value = MixerSetting (v_mas_bass, GET_SETTING);     // bass
    printf ("                            Bass   (B/b): 0x%0.2X\n", value);

    value = MixerSetting (v_mic_volume, GET_SETTING);   // microphone volume
    printf ("                            Mic    (Z/z): 0x%0.2X\n", value);

    value = MixerSetting (v_spkr_volume, GET_SETTING);  // pc speaker volume
    printf ("                            PC     (S/s): 0x%0.2X\n", value);

    value = MixerSetting (v_in_gain, GET_SETTING);      // input gain
    printf ("                            Igain  (I/i): 0x%0.2X\n", value);

    value = MixerSetting (v_out_gain, GET_SETTING);     // output gain
    printf ("                            Ogain  (O/o): 0x%0.2X\n", value);

    value = MixerSetting (v_in_agc, GET_SETTING);       // auto gain control
    printf ("                            agc    (G/g): 0x%0.2X\n", value);

    printf ("\n");


      //
      // Retrieve and display the status of various input devices using
      // MixerSetting() with the special GET_SETTING definition..
      //
    printf (" Enabled Inputs  :");
    
    value = MixerSetting (v_midi_lin_to_left, GET_SETTING);
    if (value == v_true) 
      printf ("[L_Midi]");  // midi left in to left
    else
      printf ("        ");

    value = MixerSetting (v_midi_rin_to_right, GET_SETTING);
    if (value == v_true) 
      printf ("[R_Midi]");  // midi right in to right
    else
      printf ("        ");
    
    value = MixerSetting (v_line_lin_to_left, GET_SETTING);
    if (value == v_true) 
      printf ("[L_Line]");  // line left in to left
    else
      printf ("        ");
    
    value = MixerSetting (v_line_rin_to_right, GET_SETTING);
    if (value == v_true) 
      printf ("[R_Line]");  // line right in to right
    else
      printf ("        ");
    
    value = MixerSetting (v_cd_lin_to_left, GET_SETTING);
    if (value == v_true) 
      printf ("[L_CD]");    // CD left in to left
    else
      printf ("      ");
    
    value = MixerSetting (v_cd_rin_to_right, GET_SETTING);
    if (value == v_true) 
      printf ("[R_CD]");    // CD right in to right
    else
      printf ("      ");
    
    value = MixerSetting (v_mic_in_to_left, GET_SETTING);
    if (value == v_true) 
      printf ("[L_Mic]");   // microphone in to left
    else
      printf ("       ");
    
    value = MixerSetting (v_mic_in_to_right, GET_SETTING);
    if (value == v_true) 
      printf("[R_Mic]");    // microphone in to right
    else
      printf("       ");

    printf ("\n");


      //
      // Retrieve and display the status of various output devices using
      // MixerSetting() with the special GET_SETTING definition..
      //
    printf(" Enabled Outputs :");
    
    value = MixerSetting (v_line_lout, GET_SETTING);
    if (value == v_true) 
      printf ("[L_Line]");  // line left out
    else
      printf ("        ");
    
    value = MixerSetting (v_line_rout, GET_SETTING);
    if (value == v_true) 
      printf ("[R_Line]");  // line right out
    else
      printf ("        ");
    
    value = MixerSetting (v_cd_lout, GET_SETTING);
    if (value == v_true) 
      printf ("[L_CD]");    // CD left out
    else
      printf ("      ");
    
    value = MixerSetting (v_cd_rout, GET_SETTING);
    if (value == v_true) 
      printf ("[R_CD]");    // CD right out
    else
      printf ("      ");
    
    value = MixerSetting (v_mic_out, GET_SETTING);
    if (value == v_true) 
      printf ("[Mic]");     // microphone out
    else
      printf ("     ");
    
    printf ("\n\n");


      //
      // Display information on how to toggle the input/output device status.
      //
    printf ("     To set inputs  (On/Off) : (1/!)=Mic (2/@)=CD (3/#)=Line (4/$)=Midi\n");
    printf ("     To set outputs (On/off) : (6/^)=Mic (7/&)=CD (8/*)=Line\n");
    printf ("\n");
    printf ("             Press 'Q' to Quit  -  'R' to restore initial state\n");


    if (!kbhit ())    // check for a keypress using the standard method
      continue;
    c = getch ();     // if a keypress was detected, grab it

      //
      // Scan through the valid command keys.
      //
    switch (c) {
    case 'Q':
    case 'q':
        //
        // Set the quit flag to nonzero, indicating the mixer control loop
        // should be exited.
        //
      quit = 1;
      break;
    case 'R':
    case 'r':
        //
        // Restore the previously saved mixer state using MixerState().
        //
      MixerState (initial_state);
      break;
    case 'V':
        //
        // Retrieve the current master volume setting and increment it.
        //
      value = MixerSetting (v_mas_volume, GET_SETTING);
      MixerSetting (v_mas_volume, value+1);
      break;
    case 'v':
        //
        // Retrieve the current master volume setting and decrement it.
        //
      value = MixerSetting (v_mas_volume, GET_SETTING);
      MixerSetting (v_mas_volume, value-1);
      break;
    case 'W':
        //
        // Retrieve the current wave volume setting and increment it.
        //
      value = MixerSetting (v_wave_volume, GET_SETTING);
      MixerSetting (v_wave_volume, value+1);
      break;
    case 'w':
        //
        // Retrieve the current wave volume setting and decrement it.
        //
      value = MixerSetting (v_wave_volume, GET_SETTING);
      MixerSetting(v_wave_volume, value-1);
      break;
    case 'M':
        //
        // Retrieve the current midi volume setting and increment it.
        //
      value = MixerSetting (v_midi_volume, GET_SETTING);
      MixerSetting (v_midi_volume,value+1);
      break;
    case 'm':
        //
        // Retrieve the current midi volume setting and decrement it.
        //
      value = MixerSetting (v_midi_volume, GET_SETTING);
      MixerSetting (v_midi_volume,value-1);
      break;
    case 'C':
        //
        // Retrieve the current CD volume setting and increment it.
        //
      value = MixerSetting (v_cd_volume, GET_SETTING);
      MixerSetting (v_cd_volume, value+1);
      break;
    case 'c':
        //
        // Retrieve the current CD volume setting and decrement it.
        //
      value = MixerSetting (v_cd_volume, GET_SETTING);
      MixerSetting (v_cd_volume, value-1);
      break;
    case 'L':
        //
        // Retrieve the current line volume setting and increment it.
        //
      value = MixerSetting (v_line_volume, GET_SETTING);
      MixerSetting (v_line_volume, value+1);
      break;
    case 'l':
        //
        // Retrieve the current line volume setting and decrement it.
        //
      value = MixerSetting (v_line_volume, GET_SETTING);
      MixerSetting (v_line_volume, value-1);
      break;
    case 'T':
        //
        // Retrieve the current treble volume setting and increment it.
        //
      value = MixerSetting (v_mas_treble, GET_SETTING);
      MixerSetting (v_mas_treble, value+1);
      break;
    case 't':
        //
        // Retrieve the current treble volume setting and decrement it.
        //
      value = MixerSetting (v_mas_treble, GET_SETTING);
      MixerSetting (v_mas_treble, value-1);
      break;
    case 'B':
        //
        // Retrieve the current bass volume setting and increment it.
        //
      value = MixerSetting (v_mas_bass, GET_SETTING);
      MixerSetting (v_mas_bass, value+1);
      break;
    case 'b':
        //
        // Retrieve the current bass volume setting and decrement it.
        //
      value = MixerSetting (v_mas_bass, GET_SETTING);
      MixerSetting (v_mas_bass, value-1);
      break;
    case 'Z':
        //
        // Retrieve the current microphone volume setting and increment it.
        //
      value = MixerSetting (v_mic_volume, GET_SETTING);
      MixerSetting (v_mic_volume, value+1);
      break;
    case 'z':
        //
        // Retrieve the current microphone volume setting and decrement it.
        //
      value = MixerSetting (v_mic_volume, GET_SETTING);
      MixerSetting (v_mic_volume, value-1);
      break;
    case 'S':
        //
        // Retrieve the current pc speaker volume setting and increment it.
        //
      value = MixerSetting (v_spkr_volume, GET_SETTING);
      MixerSetting (v_spkr_volume, value+1);
      break;
    case 's':
        //
        // Retrieve the current pc speaker volume setting and decrement it.
        //
      value = MixerSetting (v_spkr_volume, GET_SETTING);
      MixerSetting (v_spkr_volume, value-1);
      break;
    case 'I':
        //
        // Retrieve the current input gain setting and increment it.
        //
      value = MixerSetting (v_in_gain, GET_SETTING);
      MixerSetting (v_in_gain, value+1);
      break;
    case 'i':
        //
        // Retrieve the current input gain setting and decrement it.
        //
      value = MixerSetting (v_in_gain, GET_SETTING);
      MixerSetting (v_in_gain, value-1);
      break;
    case 'O':
        //
        // Retrieve the current output gain setting and increment it.
        //
      value = MixerSetting (v_out_gain, GET_SETTING);
      MixerSetting (v_out_gain, value+1);
      break;
    case 'o':
        //
        // Retrieve the current output gain setting and decrement it.
        //
      value = MixerSetting (v_out_gain, GET_SETTING);
      MixerSetting (v_out_gain, value-1);
      break;
    case 'G':
        //
        // Retrieve the current agc setting and enable it. This is done
        // by using a false setting, contrary to how you might expect it.
        //
      MixerSetting (v_in_agc, v_false);
      break;
    case 'g':
        //
        // Retrieve the current agc setting and disable it. This is done
        // by using a true setting, contrary to how you might expect it.
        //
      MixerSetting (v_in_agc, v_true);
      break;

      //
      // Enable or disable the microphone output status.
      //
    case '6':
      MixerSetting (v_mic_out, v_true);
      break;
    case '^':
      MixerSetting (v_mic_out, v_false);
      break;

      //
      // Enable or disable the CD output status.
      //
    case '7':
      MixerSetting (v_cd_out, v_true);
      break;
    case '&':
      MixerSetting (v_cd_out, v_false);
      break;

      //
      // Enable or disable the line output status.
      //
    case '8':
      MixerSetting (v_line_out, v_true);
      break;
    case '*':
      MixerSetting (v_line_out, v_false);
      break;

      //
      // Enable or disable the microphone input status.
      //
    case '1':
      MixerSetting (v_mic_in, v_true);
      break;
    case '!':
      MixerSetting (v_mic_in, v_false);
      break;

      //
      // Enable or disable the CD input status.
      //
    case '2':
      MixerSetting (v_cd_in, v_true);
      break;
    case '@':
      MixerSetting (v_cd_in, v_false);
      break;

      //
      // Enable or disable the line input status.
      //
    case '3':
      MixerSetting (v_line_in, v_true);
      break;
    case '#':
      MixerSetting (v_line_in, v_false);
      break;

      //
      // Enable or disable the midi input status.
      //
    case '4':
      MixerSetting (v_midi_in, v_true);
      break;
    case '$':
      MixerSetting (v_midi_in, v_false);
      break;
    };
  }


    //
    // Reset the cursor and query a restore of the initial mixer state.
    //
  setct (NORMALCURSOR);
  clrscr ();
  gotoxy (20, 10);
  printf ("Restore the mixer to its initial state? (Y/n)");
  fflush (stdout);
  c = getch ();
  gotoxy (30, 14);
  if (c != 'N' && c != 'n') {
    MixerState (initial_state);
    printf ("Mixer state restored!\n");
  }


    //
    // Shut down VAT and exit.
    //
  SBCleanUp ();
  return 0;
}

