/*_________________________________________________________________________*\
                            \/ /             \ \/
                              <  MPU Example  >
                               \_____________/

                          VARMINT'S AUDIO TOOLS 1.1


  This file is part of the VAT 1.1 distribution set. It is being provided
  as part of the VAT 1.1 demo distribution as a courtesy to prospective
  Ground Up customers. Unregistered users are granted a limited license to
  compile and run this code for demonstration purposes.  Please see the
  LICENSE.TXT file for information on how you may use the VAT library.

  This program demonstrates the VAT MPU API (Application Programming
  Interface).  Please refer to your VAT Reference Manual for information
  not covered here.  Correspondence should be directed to:

    support@groundup.com

  We very much encourage your comments and suggestions.  We want VAT to
  be useful and enjoyable for you.

    Please visit our web site at:

        http://www.groundup.com/

                                            Copyright 1995,1996 Ground Up
\*_________________________________________________________________________*/


#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "vat.h"                  // VAT's header file, always mandatory


//-------------------------------- main () --------------------------------//

int main ()
{
  INSTRUMENT *inst;  // INSTRUMENT array, used to capture default set
  int  inst_indx;    // instrument index
  int  note;         // used for note setting


  printf ("Initializing SoundBlaster at 45kHz.\n");
    //
    // Start VAT at 45Khz. As we are in a text mode the refresh setting
    // is not important; we'll use 70hz for it. If there is a problem,
    // SBSetUp() returns 0 and the sberr global variable is set to an
    // index into the errname global variable which can be used to report
    // the cause of the error.
    //
  if (!SBSetUp (45454, 70)) {
    printf ("Error in SBSetUp(): %s\n", errname[sberr]);
    exit (1);
  }
    //
    // SBSetUp() must be paired with GoVarmint() before any audio output.
    //
  GoVarmint ();
    //
    // Report the actual rate VAT was initialized at. This may be altered
    // from the rate specified above in two ways:
    //  1. The rate will be slightly adjusted to one of many predefined
    //      levels which were found to be optimal for audio output.
    //  2. If the VAT library used is the shareware or demo version, the
    //      rate will be limited to 11kHz.
    //
  printf ("Actually initialized to: %dHz\n", SampleRate ());


  printf ("Getting default instrument set...\n");
    //
    // Retrieve the instrument set in use by VAT using MidiInstruments()
    // with the special GET_INSTRUMENTS definition. Because we have not
    // activated a different instrument set, the default instrument set
    // will be returned.
    //
  inst = MidiInstruments (GET_INSTRUMENTS);


  printf ("Finding and initializing MPU-401 port...\n");
  printf ("  NOTE: if this locks, you may need to run MPUFIX.EXE\n");
    //
    // Enable MPU-401 output. If there is a problem, a value of zero is
    // returned and MPUStatus() will return v_unavail from then on.
    //
  if (!MPUEnable ()) {
    printf ("Initialization failed\n");
    exit (0);
  }


    //
    // Play a sample of each of the default instruments.
    //
  printf ("Playing six echoing notes of each default midi instrument...\n");
  inst_indx = 0; // initialize instrument index to zero
  note = 36;     // initialize the note to 36, the second 'C' note

    //
    // Loop through all 128 default instruments, playing notes 36, 48, 60,
    // 72, 84, and 96 (increments of 12 - an octave). Play several instances
    // of each note with fading volume for an echo-ish kind of effect.
    //
  while (!kbhit () && inst_indx < 128) {
      //
      // If the note is now greater than 96, it's time to move on to the
      // next instrument. Reset the note to 36 and increment inst_indx.
      //
    if (note > 96) {
      note = 36;
      inst_indx++;
    }
      //
      // Stop the previous note. This is necessary to reset some infinite
      // controllers which would otherwise obscure later samples.
      //
    if (note == 36 && inst_indx)
      MPUStopNote (0, 96);
    else if (note != 36)
      MPUStopNote (0, note-12);
      //
      // If the note is now 36, this is a new instrument. Say so and set
      // the first MPU channel's active patch to the current instrument.
      //
    if (note == 36) {
      printf ("\nPlaying instrument %d [%s]...",
              inst_indx, inst[inst_indx].name);
      fflush (stdout);
      MPUPatch (0, inst_indx);
    }
      //
      // Each of these pairs sets the first MPU channel's pitch to the
      // current note and its volume to a gradually decreasing value.
      //
    MPUPlayNote (0, note, 127);
    MilliDelay (100);
    MPUPlayNote (0, note, 112);
    MilliDelay (100);
    MPUPlayNote (0, note,  96);
    MilliDelay (100);
    MPUPlayNote (0, note,  80);
    MilliDelay (100);
    MPUPlayNote (0, note,  64);
    MilliDelay (200);

    note += 12;  // increment the note to the next octave
  }
    //
    // Again, stop the previous note.
    //
  if (note == 36 && inst_indx)
    MPUStopNote (0, 84);
  else if (note != 36)
    MPUStopNote (0, note-12);
    //
    // If the loop exited because of a keypress, grab it and throw it away.
    //
  if (kbhit ())
    getch ();


    //
    // Disable MPU output; unnecessary since this only indicates that VAT
    // should not send Midi output to the MPU-401 port, but included here
    // for demonstrative purposes. See the Midi example program for more
    // appropriate use of this.
    //
  MPUDisable ();


    //
    // Shut down VAT and exit.
    //
  printf ("\n\nShutting down VAT.\n");
  DropDeadVarmint ();
  SBCleanUp ();
  return 0;
}

