/*_________________________________________________________________________*\
                           \/ /               \ \/
                             <  Timer Example  >
                              \_______________/

                          VARMINT'S AUDIO TOOLS 1.1


  This file is part of the VAT 1.1 distribution set. It is being provided
  as part of the VAT 1.1 demo distribution as a courtesy to prospective
  Ground Up customers. Unregistered users are granted a limited license to
  compile and run this code for demonstration purposes.  Please see the
  LICENSE.TXT file for information on how you may use the VAT library.

  This program demonstrates the VAT Timer API (Application Programming
  Interface).  Please refer to your VAT Reference Manual for information
  not covered here.  Correspondence should be directed to:

    support@groundup.com

  We very much encourage your comments and suggestions.  We want VAT to
  be useful and enjoyable for you.

    Please visit our web site at:

        http://www.groundup.com/

                                            Copyright 1995,1996 Ground Up
\*_________________________________________________________________________*/


#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <graph.h>
#include "vat.h"                // VAT's header file, always mandatory

  // macros to simplify part of the example
#define gotoxy(x,y)       _settextposition(y,x)
#define clrscr()          _clearscreen(0)
#define setct(x)          _settextcursor(x)

  // definitions to simplify part of the example
#define SOLIDCURSOR   0x0007
#define NORMALCURSOR  0x0607
#define NOCURSOR      0x2000


//-------------------------------- main () --------------------------------//

int main ()
{
  WORD   ticks;           // used to hold CPU cycle measurements
  int    count;           // counting variable used in trig loop
  volatile float  total;  // summing variable used in trig loop

    //
    // Start VAT at 4Khz. As we are in a text mode the refresh setting
    // is not important; we'll use 70hz for it. Furthermore, since we're
    // not going to be performing any audio tasks, the sample rate is
    // unimportant here and we don't need to use GoVarmint(). If there is
    // a problem with SBSetUp(), it returns 0 and the sberr global variable
    // is set to an index into the errname global variable which can be
    // used to report the cause of the error.
    //
  if (!SBSetUp (4000, 70)) {
    printf ("Error in SBSetUp(): %s\n", errname[sberr]);
    exit (1);
  }


  setct (NOCURSOR);  // disable the cursor
  clrscr ();         // clear the screen


    //
    // Loop timer demo until a keypress is detected. The demo consists of
    // repeatedly timing known values and the performance of some trig
    // functions.
    //
  while (!kbhit ()) {
    gotoxy (0,0);
    printf ("                         VAT Timer Usage Examples\n\n");

      //
      // Use TimerOn() to start the VAT timer, then delay 5 milliseconds
      // and retrieve the lapsed CPU cycles since the TimerOn() call with
      // TimerOff(). Store the value in the ticks variable and convert to
      // milliseconds using the knowledge that there are approximately
      // 1,193 CPU cycles per millisecond across all Intel architectures.
      //
    TimerOn ();
    MilliDelay (5);
    ticks = TimerOff ();
    printf ("                     MilliDelay (5); "
            "lasted for %4.2fms \b\n\n", ticks / 1193.0);

      //
      // This time measure a 3,000 microsecond delay.
      //
    TimerOn ();
    MicroDelay (3000);
    ticks = TimerOff();
    printf ("                    MicroDelay (3000); "
            "lasted for %4.2fms \b\n\n", ticks / 1193.0);

      //
      // Now measure 1000 cos() function calls.
      //
    TimerOn ();
    for (count=0, total=0.0; count<1000; count++)
      total += cos ((float)count);
    ticks = TimerOff();
    printf ("                  1000 calls to cos(x); "
            "lasted for %4.2fms \b\n\n", ticks / 1193.0);

    printf ("\n\n\n\n\n\n\n                          Press a key to exit\n");

  }
  getch ();  // grab the key pressed and throw it away


  setct (NORMALCURSOR);  // reset the cursor


    //
    // Shut down VAT and exit.
    //
  SBCleanUp ();
  return 0;
}

