/**************************************************************
   This example shows how to set up a basic device and viewport
   ,set up primitives and clut and load and display a 3ds file.

	Note.
	All gifs texture maps must be in the current directory
**************************************************************/

#define	WIN_32	1
#define	_cplusplus	1

#include "..\..\..\win_h\z_all.h"

#define		NAME	"Mr DD Demo 2"

#define		SCREEN_RES_X	320
#define		SCREEN_RES_Y	200


HWND			hwnd;
int			app_active=FALSE;
int			installed=FALSE;

/* object variables */
	VECTOR	object_trans;
	SVECTOR	object_angle={2048,0,0};
	MATRIX	object_matrix;

	DEVICE_TYPE		device;
	VIEWPORT_TYPE	view;



	int shade=MAX_SHADE>>1;

	MATRIX	cmat={
					MAX_SHADE, MAX_SHADE, MAX_SHADE,
					0,0,0,
					0,0,0,
					0,0,0	/* translation (none)*/
				};

	MATRIX	light;

	DVECTOR			sxy[10000];
	short			sz[10000];

	SVECTOR			light_rotation={-1,1,-1};

	int				poly_type=MTEXTURED|MSOLID;
	char			type_string[128];
	int				model_number=0;

	char			*tpage0;
	char			*ramp_clut;
	char			*rgb_clut;
	char			*semi_clut;
	VERTEX			*next_vertex_ptr;
	POLYGON			*next_face_ptr;
	u_short			*next_index_ptr;
	int				next_vertex_no;

	MDL				model1[100];

	float			scale;
	char			file_name[256];


/*************************************************/
/* Main loop responsible for rotating models etc */
/*************************************************/
BOOL main_loop()
{
	int		t1, t2, no_of_frames, tmp, time_diff;
	MATRIX	rotlgt;
	static	SVECTOR	lgtang={0,2048,0};
	static	SVECTOR	lgtang2={0,2048,0};
	static	VECTOR	trans={0,0,0};
	char	err_str[16];
	int		err;
	static	SVECTOR	ang_vel={90,-90, 0};
	static	SVECTOR	ang_acc={-1, 1, 0};

	no_of_frames=1;

	/* Rotate light */
	if (light_rotation.vx>0)
		lgtang.vx+=32*no_of_frames;
	if (light_rotation.vy>0)
		lgtang.vy+=32*no_of_frames;
	if (light_rotation.vz>0)
		lgtang.vz+=32*no_of_frames;
	lgtang.vx&=4095;
	lgtang.vy&=4095;
	lgtang.vz&=4095;

	/* Rotate ambient light */
	SetBackColor(shade,shade,shade);

	/* Create constant light matrix */
	RotMatrixYXZ(&lgtang, &light);
	TransMatrix(&light, &trans);
	SetLightMatrix(&light);

	object_angle.vy+=ang_vel.vy;
	ang_vel.vy+=ang_acc.vy;
	if (ang_vel.vy<-90)
		ang_acc.vy=1;
	if (ang_vel.vy>90)
		ang_acc.vy=-1;

	/* Masks objects rotation */
	object_angle.vx&=4095;
	object_angle.vy&=4095;
	object_angle.vz&=4095;

//	TSC_read(&tmp, &t1);	/* Read time stamp counter */

	/* Create rotation and translation matrix for object */
	RotMatrix(&object_angle, &object_matrix);
	TransMatrix(&object_matrix, &object_trans);
	SetTransMatrix(&object_matrix);
	SetRotMatrix(&object_matrix);

	/* Draw the model */
	MDL_plot_model(&view, &model1[0], sxy, sz, poly_type);

	/* Process the device and all viewports etc.. connected to it */
	err=DEVICE_process();
	if (err!=DEVICE_OK)
	{
		itoa(err, err_str, 10);
		MessageBox(hwnd, "DEVICE ERROR", "FANCY THAT", MB_OK);
		MessageBox(hwnd, err_str, "FANCY THAT", MB_OK);

		return(FALSE);
	}

//	TSC_read(&tmp, &t2);	/* Read time stamp counter again */

	/* Calculate frame rate */
/*	time_diff=cpu_clock_speed/(t2-t1);*/
/*	no_of_frames=refresh_rate/time_diff;*/
/*	if (no_of_frames==0) no_of_frames++;*/
	frame_count++;

	return(TRUE);
}
/*****************************************/
/* WindowProc gets called every instance */
/*****************************************/
long FAR PASCAL WindowProc( HWND hWnd, UINT message, 
                            WPARAM wParam, LPARAM lParam )
{
	HDC			hdc;

	switch(message)
	{
		case WM_ACTIVATEAPP:
			app_active = wParam;

			/* If switched back to then rest palette */
			if (app_active)		
				DEVICE_set_palette(&device);
			break;

		case WM_CREATE:
			break;

		case WM_SETCURSOR:
			SetCursor(NULL);
			return TRUE;

		case WM_KEYDOWN:
		    switch( wParam )
		    {
				case VK_ESCAPE:
				case VK_F12:
				PostMessage(hWnd, WM_CLOSE, 0, 0);
				break;
		    }
		    break;

		case WM_PAINT:
			break;

		case WM_DESTROY:
			PostQuitMessage( 0 );
			break;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

/****************************************************/
/* InitWindow gets called first instance            */
/****************************************************/
/* Set up device, viewport and load in graphics etc */
/****************************************************/
static BOOL InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	WNDCLASS            wc;
	HDC                 hdc;

	installed=FALSE;


	/************************************/
	/* set up and register window class */
	/************************************/

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( hInstance, IDI_APPLICATION );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = NULL;
    wc.lpszMenuName = NAME;
    wc.lpszClassName = NAME;
    RegisterClass( &wc );

	/*******************/
	/* create a window */
	/*******************/

	hwnd = CreateWindowEx(
		WS_EX_TOPMOST,
		NAME,
		"Zeus WIN95 ex1 - Copyright brainstorm 1996",
		WS_POPUP,
		0, 0,
       GetSystemMetrics( SM_CXSCREEN ),
       GetSystemMetrics( SM_CYSCREEN ),
		NULL,
		NULL,
		hInstance,
		NULL
		);


	if (!hwnd)
	{
		return(FALSE);
	}

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);


	return(TRUE);
}
int run_command_line(char *cmd)
{
	int		t;
	int		len=strlen(cmd);
	int		index=0;
	char	fnum[256];

	if (len<2) return(-1);

	/* Get file name */
	do
	{
		file_name[index++]=*(cmd++);
		len--;
		if (len<=0) return(-1);
	}
	while (*cmd!=' ');
	file_name[index]=0;

	cmd++;
	/* Get models scale */
	index=0;
	do
	{
		fnum[index++]=*(cmd++);
		len--;
		if (len<=0) break;
	}
	while (*cmd!=' ');
	fnum[index]=0;

	scale=atof(fnum);

	if (scale==0) return(-1);

	return(0);	
}
BOOL setup_all()
{
	int					err;
	char				err_str[16];
	_RECT				view_dim;
	int					t;

	/* This is just so I can keep track of number of game frames */
	frame_count=0;

	/* Initialize zeus */
	if (SYS_init_zeus()==NULL)
	{
		MessageBox(hwnd, "Could not initialize zeus", "FANCY THAT", MB_OK);
		return(FALSE);
	}

	/* Request a texture page */
	tpage0=SYS_request_texture_page_mem(2048);
	if (tpage0==NULL)
	{
		MessageBox(hwnd, "Not enough memory for texture page", "FANCY THAT", MB_OK);
		return(FALSE);
	}

	/* Request a clut (FLAT color model) */
	ramp_clut=SYS_request_clut_mem();
	if (ramp_clut==NULL)
	{
		MessageBox(hwnd, "Not enough memory for ramp clut", "FANCY THAT", MB_OK);
		return(FALSE);
	}
	
	/* Request a clut (RGB color model), both models can be mixed in same scene */
	rgb_clut=SYS_request_clut_mem();
	if (rgb_clut==NULL)
	{
		MessageBox(hwnd, "Not enough memory for rgb clut", "FANCY THAT", MB_OK);
		return(FALSE);
	}
	
	/* Request 20,000 graphics primitives */
	if (SYS_request_n_tris((int *)&next_index_ptr, (int *)&next_face_ptr, (int *)&next_vertex_ptr, 20000)==NULL)
	{
		MessageBox(hwnd, "Not enough memory for triangles", "FANCY THAT", MB_OK);
		return(FALSE);
	}

	/* Clear device structure */
	DEVICE_clear(&device);

	/* Give window handle to device */
	device.hwnd=hwnd;

	/* Create primary device */
	err=DEVICE_create(&device, SCREEN_RES_X, SCREEN_RES_Y, 8, DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN|DDSCL_ALLOWMODEX,
			0, VIDEO_RAM_BASE_PC, DEVICE_AUTO_CLEAR, NULL, 1);
	if (err!=DEVICE_OK)
	{
		itoa(err, err_str, 10);
		MessageBox(hwnd, "DEVICE ERROR", "FANCY THAT", MB_OK);
		MessageBox(hwnd, err_str, "FANCY THAT", MB_OK);

		return(FALSE);
	}

	/* Turn vertical sync off */
	DEVICE_vertical_sync_off(&device);

	/* Load palette and set */
	DEVICE_load_palette(&device, "../newpal.pal");
	DEVICE_set_palette(&device);

	/* Create us a viewport */
	VIEWPORT_clear(&view);
	view_dim.x=0;
	view_dim.y=0;
	view_dim.w=SCREEN_RES_X-1;
	view_dim.h=SCREEN_RES_Y-1;
	err=VIEWPORT_create(&view, &view_dim, 0, FP7, 1024, DEVICE_NO_AUTO_CLEAR);
	if (err!=VIEWPORT_OK)
	{
		itoa(err, err_str, 10);
		MessageBox(hwnd, "VIEWPORT ERROR", "FANCY THAT", MB_OK);
		MessageBox(hwnd, err_str, "FANCY THAT", MB_OK);

		return(FALSE);
	}
	VIEWPORT_set_border(&view, 15);			/* Tells viewport to display a border */
	VIEWPORT_set_min_size(&view, 32,8);		/* Minimized size */
	VIEWPORT_set_max_size(&view, 320,200);	/* Maximized size */
	VIEWPORT_set_near_z(&view, 2);			/* Font Z clipping plane value */
	VIEWPORT_set_interpolation(&view, 4);	/* interpolation during perspective correct texture mapped rendering */
	VIEWPORT_set_perspective(&view, 8);		/* Set eye distance from screen */

	/* Assign various parameters to the viewport */
	VIEWPORT_assign_tpage(&view, tpage0);
	VIEWPORT_assign_ramp_clut(&view, ramp_clut);
	VIEWPORT_assign_rgb_clut(&view, rgb_clut);
	VIEWPORT_assign_semi_clut(&view, semi_clut);
	VIEWPORT_assign_prims(&view, (int *)&next_index_ptr,
		(int *)&next_face_ptr, (int *)&next_vertex_ptr, (int *)&next_vertex_no);

	/* Add the viewport to device */
	DEVICE_add_viewport(&device, &view);
				 
	/* Make 'device' the current working device */
	if (err=DEVICE_make_current(&device)<0)
	{
		itoa(err, err_str, 10);
		MessageBox( hwnd, err_str, "FANCY THAT", MB_OK );

		return(FALSE);
	}

	/* Make 'view' current working viewport */
	VIEWPORT_make_current(&view);

	/* Always call at least once before doing anything with it */
	DEVICE_process();

	installed=TRUE;

	/* Load ramp and rgb cluts */
	FILE_load_clut("../pal_ref1.tab", ramp_clut);
	FILE_load_clut("../pal_ref2.tab", rgb_clut);

	/* Set scale of 3DS models to be loaded */
	MDL_set_3ds_scale(scale, scale, scale);

	/* Clear the model structures or nasty things will happen when you are trying to delete them */
	MDL_clear(&model1[0], 100);
	/* Load 3DS model file */
	err=MDL_load_3ds_all(file_name, model1, &device, 10, tpage0, 2048, MDL_LINK_OBJECTS);
	if (err!=MDL_OK)
	{
		itoa(err, err_str, 10);
		MessageBox( hwnd, "ERROR loading 3ds file", "FANCY THAT", MB_OK );
		MessageBox( hwnd, err_str, "FANCY THAT", MB_OK );

		return(FALSE);
	}

	/* Centre object */
	MDL_centre_object(model1);

	/* Calculate models normals */
	err=MDL_calculate_normals(model1);
	if (err<0)
	{
		MessageBox( hwnd, "ERROR calculating normals", "FANCY THAT", MB_OK );

		return(FALSE);
	}

	/* Set constant color matrix */
	SetColorMatrix(&cmat);

	/* Set objects position in space */
	object_trans.vx=0;
	object_trans.vy=0;
	object_trans.vz=512;

	/* Set color to depth queue to, in this case to black */
	SetFarColor(0,0,0);

	return(TRUE);
}
void cleanup_all()
{
	/* Get rid of model and all its materials etc.. */
	MDL_kill(&model1[0]);

	/* Close down device, all viewports, order tables, primitive data, palettes etc.. */
	DEVICE_delete(&device);
}

int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                        LPSTR lpCmdLine, int nCmdShow)
{
	MSG         msg;
	int			err;

	lpCmdLine = lpCmdLine;

	/* Pull in command line options */
	if (run_command_line((char *)lpCmdLine)<0)
	{
		printf("syntax: main file.3ds 3ds_scale\n");
		return(FALSE);
	}

	/* Set up a window */
	hPrevInstance = hPrevInstance;
	if (!InitWindow(hInstance, nCmdShow))
	{
	    return(FALSE);
	}


	/* Make a device, viewport and load in graphics files etc */
	if (!(setup_all()))
	    return(FALSE);

	for (;;)
	{
		if (PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
			break;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			if (installed)
			{
				if (app_active)	/* Only run if we have not been switched away from */
				{
					if (!main_loop())
						return(FALSE);
				}
			}
		}
	}

	cleanup_all();

	return(msg.wParam);
}
