/**************************************************************
	EXPLODING BALLS

   This example shows how to set up a basic device and viewport
   ,set up a circle primitives and clut. It also shows how to
	spin them and change the graphic by adjusting the texture
	page offset. It also adds depth queuing.
**************************************************************/

#define	WIN_32	1
#define	_cplusplus	1

#include "..\..\..\win_h\z_all.h"

#define		NAME	"Mr DD Demo 1"

#define		SCREEN_RES_X	320
#define		SCREEN_RES_Y	200


#define		MAX_SPHERES		125

HWND			hwnd;
int			app_active;

/* object variables */
	VECTOR			object_trans={0,0,2048+512};
	SVECTOR			object_angle={0,0,0};
	MATRIX			object_matrix;

/* Device and viewport */
	DEVICE_TYPE		device;
	VIEWPORT_TYPE	view;

	char			*ramp_clut;

	CIRCLE			circles[MAX_SPHERES];

	SVECTOR			circle_centres[MAX_SPHERES];
	SVECTOR			old_circle_centres[MAX_SPHERES];

	SVECTOR			circle_normals[MAX_SPHERES];

	SVECTOR			circle_vels[MAX_SPHERES];

/* light source */
	MATRIX	cmat={
					MAX_SHADE, MAX_SHADE, MAX_SHADE,
					0,0,0,
					0,0,0,
					0,0,0	/* translation (none)*/
				};

	MATRIX	light;
	SVECTOR	lgtang={0,3072,0};

	int				light_fuse=0;
	int	 			explosion_on=-1;
	int				max_frames;


void create_centres(SVECTOR *trans, SVECTOR *normals, int num_spheres)
{
	int		t;
	int		x, y, z;
	int		ox, oy, oz;
	int		dx, dy, dz;
	int		mx, my, mz;

	dx=256;
	dy=256;
	dz=256;

	x=-dx*2;
	y=-dy*2;
	z=-dz*2;

	ox=x;
	oy=y;
	oz=z;

	mx=dx*2;
	my=dy*2;
	mz=dz*2;

	for (t=0; t<num_spheres; t++)
	{	
		/* Create a random translation in space */
		trans->vx=x;
		trans->vy=y;
		trans->vz=z;

		x+=dx;
		if (x>mx)
		{
			x=ox;
			y+=dy;
			if (y>my)
			{
				y=oy;
				z+=dz;
			}
		}

		normals->vx=0;
		normals->vy=0;
		normals->vz=ONE;

		trans++;
		normals++;
	}
}

void calculate_explosion(SVECTOR *vel, int num_spheres)
{
	int		t;

	for (t=0; t<num_spheres; t++)
	{	
		/* Create a random translation in space */
		vel->vx=((rand()-16384) >> 8);
		vel->vy=((rand()-16384) >> 8);
		vel->vz=((rand()-16384) >> 8);

		vel++;
	}
}

void fuse_explosion(int num_frames)
{
	/* Reset positions of balls */
	memcpy((void *)old_circle_centres, (void *)circle_centres, sizeof(SVECTOR) * MAX_SPHERES);
	calculate_explosion(&circle_vels[0], MAX_SPHERES);
	max_frames=num_frames;
	light_fuse=num_frames;
	explosion_on=1;
}

void draw_spheres(SVECTOR *circle_trans, SVECTOR *normal_list)
{
	int			t, ot_pos;
	int			tpage;
	VECTOR		ov;
	long		p, flag;
	SVECTOR		r_normal;	/* Rotated normal */
	CVECTOR		colors={MAX_SHADE/2, MAX_SHADE/2, MAX_SHADE/2};	/* Base shade */
	SVECTOR		trans;

	for (t=0; t<MAX_SPHERES; t++)
	{
		/* Takes care of balls position when exploding */
		if (explosion_on>0)
		{
			if (light_fuse>0)	/* expand balls */
			{
				light_fuse--;
				if (light_fuse==0) light_fuse=-max_frames;

				trans.vx=old_circle_centres[t].vx;
				trans.vy=old_circle_centres[t].vy;
				trans.vz=old_circle_centres[t].vz;
				old_circle_centres[t].vx+=circle_vels[t].vx;
				old_circle_centres[t].vy+=circle_vels[t].vy;
				old_circle_centres[t].vz+=circle_vels[t].vz;
			}
			else
				if (light_fuse<0)	/* retract balls */
				{
					light_fuse++;
					if (light_fuse==0) explosion_on=-1;

					trans.vx=old_circle_centres[t].vx;
					trans.vy=old_circle_centres[t].vy;
					trans.vz=old_circle_centres[t].vz;
					old_circle_centres[t].vx-=circle_vels[t].vx;
					old_circle_centres[t].vy-=circle_vels[t].vy;
					old_circle_centres[t].vz-=circle_vels[t].vz;
				}
		}
		/* Takes care of balls position when not exploding */
		else	
		{
			trans.vx=circle_trans->vx;
			trans.vy=circle_trans->vy;
			trans.vz=circle_trans->vz;
		}
		/* spin sphere using texture page start and change texture */
		tpage=((frame_count<<1)&127) + (t&3)*100*TPAGE_WIDTH;

		/* Rotate translate and add perspective to spheres centre */
		RotTrans(&trans, &ov, &flag);

		/* Need this for depth queuing */
		p=ov.vz>>1;
		if (p<1) p=1;

		/* Scale order table position down a little so we don't drop off end */
		ot_pos=ov.vz>>2;

		/* Only draw if sphere is in front of viewer */
		if (ov.vz>VIEWPORT_current->z0)
		{
			/* Dont need to rotate the normal since we always want it to point at viewer */
			/* Calculate shade of sphere (with depth queuing) */
			NormalColorDpq(normal_list, &colors, p, (CVECTOR *)&(circles[t].r));

			/* set up circle structure */
			circles[t].tag=NULL;
			circles[t].id=MSPHERE|MTEXTURED;

			/* If sphere is quite big then we can switch prim to double pixel mode */
			if (ot_pos<64)
				circles[t].id|=MX2;

			/* Setup spheres radius */
			circles[t].xr=128;
			circles[t].yr=128;

			/* Set up sheres 3D position */
			circles[t].cx=ov.vx;
			circles[t].cy=ov.vy;
			circles[t].cz=ov.vz;
			circles[t].c=15;

			/* Set up texture coardinates */
			circles[t].u[0]=0;
			circles[t].u[1]=FP_TEXT(127);
			circles[t].u[2]=FP_TEXT(127);
			circles[t].u[3]=0;
			circles[t].v[0]=0;
			circles[t].v[1]=0;
			circles[t].v[2]=FP_TEXT(100);
			circles[t].v[3]=FP_TEXT(100);
			circles[t].c=15;
			circles[t].tpage=tpage;

			/* Add primitive to viewport */
			VIEWPORT_add_prim(&view, (POLYGON *)&circles[t], ot_pos);
		}

		circle_trans++;
	}
}

int device_callback()
{
	frame_count++;

	return(NULL);
}


/*************************************************/
/* Main loop responsible for rotating models etc */
/*************************************************/
BOOL main_loop()
{
	char	err_str[16];
	int		err;

	object_angle.vx+=32;
	object_angle.vx&=4095;
	object_angle.vy+=32;
	object_angle.vy&=4095;
	object_angle.vz+=32;
	object_angle.vz&=4095;

	/* Create constant light matrix */
	RotMatrixYXZ(&lgtang, &light);
	SetLightMatrix(&light);

	/* Create rotation and translation matrix for object */
	RotMatrix(&object_angle, &object_matrix);
	TransMatrix(&object_matrix, &object_trans);

	/* Set as constant rotation and translation matrix */
	SetTransMatrix(&object_matrix);
	SetRotMatrix(&object_matrix);

	/* Draw the object */
	draw_spheres(&circle_centres[0], &circle_normals[0]);

	/* Process the device and all viewports etc.. connected to it */
	err=DEVICE_process();
	if (err!=DEVICE_OK)
	{
		itoa(err, err_str, 10);
		MessageBox(hwnd, "LOOP DEVICE ERROR", "FANCY THAT", MB_OK);
		MessageBox(hwnd, err_str, "FANCY THAT", MB_OK);

		return(FALSE);
	}

	return(TRUE);
}
/*****************************************/
/* WindowProc gets called every instance */
/*****************************************/
long FAR PASCAL WindowProc( HWND hWnd, UINT message, 
                            WPARAM wParam, LPARAM lParam )
{
	HDC			hdc;

	switch( message )
	{
		case WM_ACTIVATEAPP:
			app_active = wParam;

			/* If switched back to then rest palette */
			if (app_active)		
				DEVICE_set_palette(&device);

			break;

		case WM_CREATE:
			break;

		case WM_SETCURSOR:
			SetCursor(NULL);
			return TRUE;

		case WM_KEYDOWN:
		    switch( wParam )
		    {
				case VK_UP:
					object_trans.vz+=64;
					break;
				case VK_DOWN:
					object_trans.vz-=64;
					break;
				case VK_SPACE:
					fuse_explosion(16384); /* to explode over 16384 frames */
					break;
				case VK_ESCAPE:
				case VK_F12:
				PostMessage(hWnd, WM_CLOSE, 0, 0);
				break;
		    }
		    break;

		case WM_PAINT:
			break;

		case WM_DESTROY:
			PostQuitMessage( 0 );
			break;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

/****************************************************/
/* InitWindow gets called first instance            */
/****************************************************/
/* Set up device, viewport and load in graphics etc */
/****************************************************/
static BOOL InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	WNDCLASS            wc;
	HDC                 hdc;

	/************************************/
	/* set up and register window class */
	/************************************/

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( hInstance, IDI_APPLICATION );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = NULL;
    wc.lpszMenuName = NAME;
    wc.lpszClassName = NAME;
    RegisterClass( &wc );

	/*******************/
	/* create a window */
	/*******************/

	hwnd = CreateWindowEx(
		WS_EX_TOPMOST,
		NAME,
		"Zeus WIN95 ex1 - Copyright brainstorm 1996",
		WS_POPUP,
		0, 0,
       GetSystemMetrics( SM_CXSCREEN ),
       GetSystemMetrics( SM_CYSCREEN ),
		NULL,
		NULL,
		hInstance,
		NULL
		);


	if (!hwnd)
	{
		return(FALSE);
	}

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	return(TRUE);
}
BOOL setup_all()
{
	int					err;
	char				err_str[16];
	_RECT				view_dim;
	int					t;

	/* This is just so I can keep track of number of game frames */
	frame_count=0;

	/* Initialize zeus */
	if (SYS_init_zeus()==NULL)
	{
		MessageBox(hwnd, "Could not initialize zeus", "FANCY THAT", MB_OK);
		return(FALSE);
	}

	/* Request a clut (FLAT color model) */
	ramp_clut=SYS_request_clut_mem();
	if (ramp_clut==NULL)
	{
		MessageBox(hwnd, "Not enough memory for ramp clut", "FANCY THAT", MB_OK);
		return(FALSE);
	}
	
	tpage0=SYS_request_texture_page_mem(2*256);
	if (tpage0==NULL)
	{
		printf("Not enough memory for texture page\n");
		exit(1);
	}

	/* Clear device structure */
	DEVICE_clear(&device);

	/* Give window handle to device */
	device.hwnd=hwnd;

	/* Create primary device */
	err=DEVICE_create(&device, SCREEN_RES_X, SCREEN_RES_Y, 8, DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN|DDSCL_ALLOWMODEX,
			0, VIDEO_RAM_BASE_PC, DEVICE_AUTO_CLEAR, NULL, 1);
	if (err!=DEVICE_OK)
	{
		itoa(err, err_str, 10);
		MessageBox(hwnd, "DEVICE ERROR", "FANCY THAT", MB_OK);
		MessageBox(hwnd, err_str, "FANCY THAT", MB_OK);

		return(FALSE);
	}

	/* Turn vertical sync off */
	DEVICE_vertical_sync_off(&device);

	DEVICE_set_callback(&device, &device_callback);

	/* Load palette and set */
	DEVICE_load_palette(&device, "../newpal.pal");
	DEVICE_set_palette(&device);

	/* Create us a viewport */
	VIEWPORT_clear(&view);
	view_dim.x=0;
	view_dim.y=0;
	view_dim.w=SCREEN_RES_X-1;
	view_dim.h=SCREEN_RES_Y-1;
	err=VIEWPORT_create(&view, &view_dim, 0, FP7, 1024, DEVICE_NO_AUTO_CLEAR);
	if (err!=VIEWPORT_OK)
	{
		itoa(err, err_str, 10);
		MessageBox(hwnd, "VIEWPORT ERROR", "FANCY THAT", MB_OK);
		MessageBox(hwnd, err_str, "FANCY THAT", MB_OK);

		return(FALSE);
	}
	VIEWPORT_set_border(&view, 15);			/* Tells viewport to display a border */
	VIEWPORT_set_min_size(&view, 32,8);		/* Minimized size */
	VIEWPORT_set_max_size(&view, 320,200);	/* Maximized size */
	VIEWPORT_set_near_z(&view, 0);			/* Font Z clipping plane value */
	VIEWPORT_set_interpolation(&view, 4);	/* interpolation during perspective correct texture mapped rendering */
	VIEWPORT_set_perspective(&view, 8);		/* Set eye distance from screen */

	/* Assign texture page to viewport */
	VIEWPORT_assign_tpage(&view, tpage0);

	/* Assign clut to viewport */
	VIEWPORT_assign_ramp_clut(&view, ramp_clut);

	/* Add the viewport to device */
	DEVICE_add_viewport(&device, &view);
				 
	/* Make 'device' the current working device */
	if (err=DEVICE_make_current(&device)<0)
	{
		itoa(err, err_str, 10);
		MessageBox( hwnd, err_str, "FANCY THAT", MB_OK );

		return(FALSE);
	}

	/* Make 'view' current working viewport */
	VIEWPORT_make_current(&view);

	/* Always call at least once before doing anything with it */
	DEVICE_process();

	/* Load texture raw date */
	FILE_load_tpage("../planets.bin", tpage0);

	/* Load ramp clut */
	FILE_load_clut("../pal_ref1.tab", ramp_clut);

	/* Create random sphere centres */
	create_centres(&circle_centres[0], &circle_normals[0], MAX_SPHERES);

	/* Set constant color matrix */
	SetColorMatrix(&cmat);

	/* Set color to depth queue to, in this case to black */
	SetBackColor(MAX_SHADE, MAX_SHADE, MAX_SHADE);

	/* Set color to depth queue to, in this case to black */
	SetFarColor(0,0,0);

	return(TRUE);
}
void cleanup_all()
{
	/* Close down device, all viewports, order tables, primitive data, palettes etc.. */
	DEVICE_delete(&device);
}

int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                        LPSTR lpCmdLine, int nCmdShow)
{
	MSG         msg;
	int			err;
	BOOL		installed=FALSE;

	lpCmdLine = lpCmdLine;

	/* Set up a window */
	hPrevInstance = hPrevInstance;
	if (!InitWindow(hInstance, nCmdShow))
	{
	    return(FALSE);
	}


	/* Make a device, viewport and load in graphics files etc */
	if (!(setup_all()))
	    return(FALSE);

	installed=TRUE;

	for (;;)
	{
		if (PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
			break;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			if (installed)
			{
				if (app_active)	/* Only run if we have not been switched away from */
				{
					if (!main_loop())
						return(FALSE);
				}
			}
		}
	}

	cleanup_all();

	return(msg.wParam);
}
